--{{{  Display routines
--{{{  Display Inquiry Data
PROC Display.Inquiry.Data(CHAN OF SP fs, ts,  []BYTE Iq.Buffer)
  SEQ
    so.write.nl(fs, ts)
    so.write.string.nl(fs, ts, "Inquiry Data")
    so.write.string.nl(fs, ts, "============")
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Peripheral device type         : ")
    so.write.hex.int(fs, ts, ((INT Iq.Buffer[0]) /\
                          Inquiry.6.Peripheral.Device.Type), 2)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Peripheral qualifier           : ")

    INT Peripheral.Qualifier:
    SEQ
      Peripheral.Qualifier := INT Iq.Buffer[0]
      Peripheral.Qualifier := (ROTATERIGHT (Peripheral.Qualifier, 5) /\
                              Inquiry.6.Peripheral.Qualifier)
      so.write.hex.int(fs, ts, Peripheral.Qualifier, 1)

    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Device type qualifier          : ")
    so.write.hex.int(fs, ts, ((INT Iq.Buffer[1]) /\
                          Inquiry.6.Device.type.Qualifier), 2)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Removable Medium               : ")
    so.write.hex.int(fs, ts, (ROTATERIGHT (INT Iq.Buffer[1], 7) /\
                          Inquiry.6.RMB), 1)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "ANSI-Approved Version          : ")
    so.write.hex.int(fs, ts, ((INT Iq.Buffer[2]) /\
                          Inquiry.6.ANSI.Approved.Version), 1)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "ECMA Version                   : ")
    so.write.hex.int(fs, ts, (ROTATERIGHT (INT Iq.Buffer[2], 3) /\
                          Inquiry.6.ECMA.Version), 1)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "ISO Version                    : ")
    so.write.hex.int(fs, ts, (ROTATERIGHT (INT Iq.Buffer[2], 6) /\
                          Inquiry.6.ISO.Version), 1)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Response data format           : ")
    so.write.hex.int(fs, ts, ((INT Iq.Buffer[3]) /\
                          Inquiry.6.Response.Data.Format), 1)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Terminate I/O process support  : ")
    so.write.hex.int(fs, ts, (ROTATERIGHT (INT Iq.Buffer[3], 6) /\
                          Inquiry.6.TrmIOP), 1)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "AENC                           : ")
    so.write.hex.int(fs, ts, (ROTATERIGHT (INT Iq.Buffer[3], 7) /\
                          Inquiry.6.AENC), 1)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Additional Length              : ")
    so.write.hex.int(fs, ts, INT Iq.Buffer[4], 3)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Byte 5                         : ")
    so.write.hex.int(fs, ts, INT Iq.Buffer[5], 2)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Byte 6                         : ")
    so.write.hex.int(fs, ts, INT Iq.Buffer[6], 2)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Byte 7                         : ")
    so.write.hex.int(fs, ts, INT Iq.Buffer[7], 2)
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Vendor Identification          : ")
    SEQ i = 8 FOR 8
      so.write.char(fs, ts, Iq.Buffer[i])
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Product Identification         : ")
    SEQ i = 16 FOR 16
      so.write.char(fs, ts, Iq.Buffer[i])
    so.write.nl(fs, ts)
    so.write.string(fs, ts, "Product Revison Level          : ")
    SEQ i = 32 FOR (Inquiry.Allocation.Length - 32)
      so.write.char(fs, ts, Iq.Buffer[i])
    so.write.nl(fs, ts)
:
--}}}
--}}}
--{{{  All Device Types
PROC All.Device.Types(CHAN OF SP fs, ts,
                      CHAN OF TO.SCSI tscsi,
                      CHAN OF FROM.SCSI fscsi)
  BOOL Going:
  INT Selected.Option:
  [1024]BYTE Mode.Data:
  SEQ
    Going := TRUE
    WHILE Going
      SEQ
        --{{{  Prompt User
        Display.Options(fs, ts,
                        Main.Title,
                       "All Device Types (Chapter 7)",
                        ["Exit               ",
                         "Define Bus ID      ",
                         "Request Sense      ",
                         "Test Unit Ready    ",
                         "Inquiry            ",
                         "Mode Sense         ",
                         "Mode Select        ",
                         "Mode Sense & Select"],
                         Status.Message,
                         Selected.Option)
        --}}}
        --{{{  Do User request
        CASE Selected.Option
          0
            Going := FALSE
          1
            --{{{  Prompt operator for initiator and target SCSI device ID's
            BOOL First.Time:
            SEQ
              First.Time := TRUE
              Initiator.ID := 7 (BYTE)
              Target.ID := 7 (BYTE)
              WHILE (Initiator.ID = Target.ID)
                SEQ
                  IF
                    NOT First.Time
                      --{{{  Warning Message
                      so.write.string.nl(fs, ts, "Initiator ID and Target ID must be different.")
                      --}}}
                    TRUE
                      First.Time := FALSE
                  --{{{  Prompt Operator for Initiator ID
                  INT reply:
                  SEQ
                    Quit := TRUE
                    so.ask(fs, ts,
                           "Which Initiator ID do you wish to set IMS B422 as > ",
                           "01234567",
                           TRUE,
                           TRUE,
                           reply)
                    Initiator.ID := BYTE reply
                    so.write.nl(fs, ts)
                    so.write.nl(fs, ts)
                  --}}}
                  --{{{  Prompt Operator for Target ID
                  INT reply:
                  SEQ
                    Quit := TRUE
                    so.ask(fs, ts,
                           "Which Target ID do you wish IMS B422 to talk to   > ",
                           "01234567",
                           TRUE,
                           TRUE,
                           reply)
                    Target.ID := BYTE reply
                    so.write.nl(fs, ts)
                    so.write.nl(fs, ts)
                  --}}}
              --{{{  Make IMS B422 an Initiator
              so.write.string.nl(fs, ts, "Make B422 an Initiator")
              Enable.Initiator.Mode( tscsi,
                                     fscsi,
                                     BYTE Initiator.ID,
                                     Execution.Status)
              --}}}
              Prepare.Status.Message(Execution.Status, Status.Message)
            --}}}
          2
            SEQ
              --{{{  Request Sense
              Request.Sense(fs, ts, tscsi, fscsi)
              --}}}
          3
            SEQ
              --{{{  Test Unit Ready
              so.write.string.nl(fs, ts, "SCSI.Test.Unit.Ready.6")
              Set.Identify.Message()
              
              SCSI.Test.Unit.Ready.6 ( tscsi,
                                       fscsi,
                                       Target.ID,
                                       LUN,
                                       Non.Linked.Command,
                                       Msg.Length,
                                       Message,
                                       SCSI.Status,
                                       Execution.Status)
              
              Prepare.Status.Message(Execution.Status, Status.Message)
              --}}}
          4
            SEQ
              --{{{  Inquiry
              --{{{  Null Inquiry Data
              SEQ i = 0 FOR SIZE Inquiry.Data
                Inquiry.Data[i] := Zero.B
              --}}}
              so.write.string(fs, ts, "SCSI.Inquiry.6")
              VAL BYTE EVDP      IS Zero.B:
              VAL BYTE Page.Code IS Zero.B:
              SEQ
                Set.Identify.Message()
                SCSI.Inquiry.6 ( tscsi,
                                 fscsi,
                                 Target.ID,
                                 LUN,
                                 EVDP,
                                 Page.Code,
                                 BYTE Inquiry.Allocation.Length,
                                 Non.Linked.Command,
                                 Inquiry.Data,
                                 Msg.Length,
                                 Message,
                                 SCSI.Status,
                                 Execution.Status)
              
              Prepare.Status.Message(Execution.Status, Status.Message)
              Display.Inquiry.Data( fs, ts, Inquiry.Data )
              so.write.string.nl(fs, ts, Status.Message)
              Press.Any.Key.To.Continue(fs, ts)
              --}}}
          5
            SEQ
              --{{{  Mode Sense
              so.write.string.nl(fs, ts, "SCSI.Mode.Sense.6")
              BOOL error:
              BYTE DBD:
              BYTE PC:
              INT Page.Code:
              INT reply:
              INT Allocation.Length:
              INT Selected.Option:
              SEQ
                --{{{  Null Mode Data
                SEQ i = 0 FOR SIZE Mode.Data
                  Mode.Data[i] := Zero.B
                --}}}
                --{{{  DBD
                so.ask(fs, ts, "Return block descriptors > ", "NY", TRUE, TRUE, reply)
                DBD := BYTE reply
                --}}}
                --{{{  Page Code Field
                Display.Options(fs, ts,
                                Main.Title,
                               "Page Control Field",
                                ["Current Values     ",
                                 "Changeable Values  ",
                                 "Default Values     ",
                                 "Saved Values       "],
                                  Status.Message,
                                  Selected.Option)
                PC := BYTE Selected.Option
                --}}}
                --{{{  Page Code
                error := TRUE
                WHILE error
                  SEQ
                    so.write.string(fs, ts, "Page Code [0..3F] > ")
                    so.read.echo.any.int(fs, ts, Page.Code, error)
                    so.write.nl(fs, ts)
                --}}}
                --{{{  Allocation Length
                error := TRUE
                WHILE error
                  SEQ
                    so.write.string(fs, ts, "Allocation Length > ")
                    so.read.echo.any.int(fs, ts, Allocation.Length, error)
                    so.write.nl(fs, ts)
                --}}}
                --{{{  Mode Sense
                Set.Identify.Message()
                SCSI.Mode.Sense.6 ( tscsi,
                                    fscsi,
                                    Target.ID,
                                    LUN,
                                    DBD,
                                    PC,
                                    BYTE Page.Code,
                                    BYTE Allocation.Length,
                                    Non.Linked.Command,
                                    Mode.Data,
                                    Msg.Length,
                                    Message,
                                    SCSI.Status,
                                    Execution.Status)
                Prepare.Status.Message(Execution.Status, Status.Message)
                --}}}
                --{{{  Hex Editor
                IF
                  (Execution.Status = SCSI.E.Good)
                    Hex.Editor( fs, ts, 0, Mode.Data, "Mode Sense Data")
                  TRUE
                    Hex.Editor( fs, ts, 0, Mode.Data, Status.Message)
                --}}}
              --}}}
          6
            SEQ
              --{{{  Mode Select
              so.write.string.nl(fs, ts, "SCSI.Mode.Select.6")
              VAL BYTE DBD       IS Zero.B:
              VAL BYTE PC        IS Zero.B:
              VAL BYTE Page.Code IS Zero.B:
              BYTE PF:
              BYTE SP:
              BOOL error:
              INT Allocation.Length, reply:
              SEQ
                --{{{  Null Mode Data
                SEQ i = 0 FOR SIZE Mode.Data
                  Mode.Data[i] := Zero.B
                --}}}
                error := TRUE
                WHILE error
                  SEQ
                    so.write.string(fs, ts, "Allocation Length > ")
                    so.read.echo.any.int(fs, ts, Allocation.Length, error)
                    so.write.nl(fs, ts)
                Hex.Editor( fs, ts, 0, Mode.Data, "Mode Select Data")
                --{{{  Mode Select
                --{{{  PF
                so.ask(fs, ts, "Page Format > ", "01", TRUE, TRUE, reply)
                PF := BYTE reply
                --}}}
                --{{{  SP
                so.ask(fs, ts, "Save Parameters > ", "01", TRUE, TRUE, reply)
                SP := BYTE reply
                --}}}
                Set.Identify.Message()
                SCSI.Mode.Select.6 ( tscsi,
                                     fscsi,
                                     Target.ID,
                                     LUN,
                                     PF,
                                     SP,
                                     BYTE Allocation.Length,
                                     Non.Linked.Command,
                                     Mode.Data,
                                     Msg.Length,
                                     Message,
                                     SCSI.Status,
                                     Execution.Status)
                --}}}
                Prepare.Status.Message(Execution.Status, Status.Message)
              --}}}
          7
            SEQ
              --{{{  Mode Sense / Select
              so.write.string.nl(fs, ts, "SCSI.Mode.Sense.6")
              BOOL error:
              BYTE DBD:
              BYTE PC:
              BYTE PF:
              BYTE SP:
              INT Page.Code:
              INT reply:
              INT Allocation.Length:
              INT Selected.Option:
              SEQ
                --{{{  Null Mode Data
                SEQ i = 0 FOR SIZE Mode.Data
                  Mode.Data[i] := Zero.B
                --}}}
                --{{{  DBD
                so.ask(fs, ts, "Return block descriptors > ", "NY", TRUE, TRUE, reply)
                DBD := BYTE reply
                --}}}
                --{{{  Page Code Field
                Display.Options(fs, ts,
                                Main.Title,
                               "Page Control Field",
                                ["Current Values     ",
                                 "Changeable Values  ",
                                 "Default Values     ",
                                 "Saved Values       "],
                                  Status.Message,
                                  Selected.Option)
                PC := BYTE Selected.Option
                --}}}
                --{{{  Page Code
                error := TRUE
                WHILE error
                  SEQ
                    so.write.string(fs, ts, "Page Code [0..3F] > ")
                    so.read.echo.any.int(fs, ts, Page.Code, error)
                    so.write.nl(fs, ts)
                --}}}
                --{{{  Allocation Length
                error := TRUE
                WHILE error
                  SEQ
                    so.write.string(fs, ts, "Allocation Length > ")
                    so.read.echo.any.int(fs, ts, Allocation.Length, error)
                    so.write.nl(fs, ts)
                --}}}
                --{{{  Mode Sense
                Set.Identify.Message()
                SCSI.Mode.Sense.6 ( tscsi,
                                    fscsi,
                                    Target.ID,
                                    LUN,
                                    DBD,
                                    PC,
                                    BYTE Page.Code,
                                    BYTE Allocation.Length,
                                    Non.Linked.Command,
                                    Mode.Data,
                                    Msg.Length,
                                    Message,
                                    SCSI.Status,
                                    Execution.Status)
                Prepare.Status.Message(Execution.Status, Status.Message)
                --}}}
                --{{{  Hex Editor
                IF
                  (Execution.Status = SCSI.E.Good)
                    Hex.Editor( fs, ts, 0, Mode.Data, "Mode Sense Data")
                  TRUE
                    Hex.Editor( fs, ts, 0, Mode.Data, Status.Message)
                --}}}
                --{{{  Mode Select
                Clear.Screen(fs, ts)
                so.ask(fs, ts,
                       "Issue Mode Select Command [Y|N] ? ",
                       "YN",
                       TRUE,
                       TRUE,
                       reply)
                so.write.nl(fs, ts)
                IF
                  reply = 0
                    --{{{  Mode Select
                    SEQ
                      --{{{  PF
                      so.ask(fs, ts, "Page Format > ", "01", TRUE, TRUE, reply)
                      PF := BYTE reply
                      --}}}
                      --{{{  SP
                      so.ask(fs, ts, "Save Parameters > ", "01", TRUE, TRUE, reply)
                      SP := BYTE reply
                      --}}}
                      Set.Identify.Message()
                      SCSI.Mode.Select.6 ( tscsi,
                                           fscsi,
                                           Target.ID,
                                           LUN,
                                           PF,
                                           SP,
                                           BYTE Allocation.Length,
                                           Non.Linked.Command,
                                           Mode.Data,
                                           Msg.Length,
                                           Message,
                                           SCSI.Status,
                                           Execution.Status)
                      Prepare.Status.Message(Execution.Status, Status.Message)
                    --}}}
                  TRUE
                    SKIP
                --}}}
              --}}}
          ELSE
            so.write.string.nl(fs, ts, "Not Implemented")
        --}}}
:
--}}}
