--{{{  Header
--------------------------------------------------------------------------------
--
-- Security    : Copyright 1990, 1991 INMOS Limited
--
-- Author      : Paul Fellows
--
-- Date        : 19-Jan-1990
--
-- File Name   : COMMANDS.INC
--
-- Project     : F002C - SCSI Tram Software
--
-- Version     : 0.1
--
-- Division    : IQ Systems
--
-- Limitations : This software may be distributed in source form.
--
-- Description : Hardware definitions for B422 SCSI TRAM
--
--------------------------------------------------------------------------------
-- Schedule of Amendments
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--}}}  
--{{{  Generic bits
--{{{  Disconnect
PROC Disconnect.From.Bus(BYTE Bus.Connection.State, INT16 Execution.Status)
  SEQ
    IF
      (Bus.Connection.State = Disconnected.State)
        SCSI.T.Disconnect.From.Bus( TO.TARGET.CONTROLLER,
                                    FROM.TARGET.CONTROLLER,
                                    Execution.Status)
      TRUE
        SKIP
:
--}}}  
--{{{  Reconnect
PROC Reconnect(BYTE Bus.Connection.State, Initiator.ID, INT16 Execution.Status)
  SEQ
    IF
      (Bus.Connection.State = Disconnected.State)
        SCSI.T.Reconnect.To.Bus( TO.TARGET.CONTROLLER,
                                 FROM.TARGET.CONTROLLER,
                                 Initiator.ID,
                                 Execution.Status)
      TRUE
        SKIP
:
--}}}  
--{{{  Unsupported Command
PROC Unsupported.SCSI.Command(BYTE SCSI.Status, INT16 Execution.Status)
  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  --}}}  
  --{{{  Implementation
  SEQ
  
    Msg.Length := 1 (BYTE)
    Message[0] := BYTE Terminate.IO.Process
  
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   SCSI.Non.Linked.Command,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
  
    Display.Execution.Status(Execution.Status)
  --}}}  
:
--}}}  
--}}}  
--{{{  6 Byte Commands
--{{{  Inquiry
PROC Perform.Inquiry.6( []BYTE     SCSI.Command,
                        BYTE       Bus.Connection.State,
                        BYTE       Initiator.ID,
                        VAL []BYTE Inquiry.Data,
                        []BYTE     Sense.Data,
                        BYTE       SCSI.Status,
                        INT16      Execution.Status)

  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  VAL BYTE Allocation.Length RETYPES SCSI.Command[4]:
  VAL BYTE Control.Byte RETYPES SCSI.Command[5]:
  --}}}  
  --{{{  Implementation
  SEQ
    --DEBUG.MESSAGE("Inquiry")
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    --}}}  
    Sense.Data[2] := 0 (BYTE)
    --{{{  Data In Phase
    SCSI.T.Data.In.Phase ( TO.TARGET.CONTROLLER,
                           FROM.TARGET.CONTROLLER,
                           INT32 Allocation.Length,
                           Inquiry.Data,
                           Execution.Status)
    Display.Execution.Status(Execution.Status)
    --}}}  
    --{{{  Status & Message In
    --{{{  Prepare Message
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
    --}}}  
    --{{{  Send Status and Message
    SCSI.Status := BYTE SCSI.Status.Good
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    Display.Execution.Status(Execution.Status)
    --}}}  
    --}}}  
  --}}}  
:
--}}}  
--{{{  Test Unit Ready
PROC Perform.Test.Unit.Ready.6([]BYTE  SCSI.Command,
                               BYTE    Bus.Connection.State,
                               BYTE    Initiator.ID,
                               []BYTE  Sense.Data,
                               BYTE    SCSI.Status,
                               INT16   Execution.Status)

  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  VAL BYTE Control.Byte RETYPES SCSI.Command[5]:
  --}}}  
  --{{{  Implementation
  SEQ
  
    --DEBUG.MESSAGE("Test Unit Ready")
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    Sense.Data[2] := 0 (BYTE)
  
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
  
    SCSI.Status := BYTE SCSI.Status.Good
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    Display.Execution.Status(Execution.Status)
  
  --}}}  
:
--}}}  
--{{{  Rezero Unit
PROC Perform.Rezero.Unit.6 ( []BYTE  SCSI.Command,
                             BYTE    Bus.Connection.State,
                             BYTE    Initiator.ID,
                             []BYTE  Sense.Data,
                             BYTE    SCSI.Status,
                             INT16   Execution.Status)

  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  VAL BYTE Control.Byte RETYPES SCSI.Command[5]:
  --}}}  
  --{{{  Implementation
  SEQ
  
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
  
    Sense.Data[2] := 0 (BYTE)
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
  
    SCSI.Status := BYTE SCSI.Status.Good
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
  
    Display.Execution.Status(Execution.Status)
  --}}}  
:
--}}}  
--{{{  Start Stop
PROC Perform.Start.Stop.Unit.6([]BYTE  SCSI.Command,
                               BYTE    Bus.Connection.State,
                               BYTE    Initiator.ID,
                               []BYTE  Sense.Data,
                               BYTE    SCSI.Status,
                               INT16   Execution.Status)

  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  VAL BYTE Control.Byte RETYPES SCSI.Command[5]:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    --}}}  
  
    Sense.Data[2] := 0 (BYTE)
    Msg.Length := 1 (BYTE)
    Message[0] := BYTE Command.Complete
  
    SCSI.Status := BYTE SCSI.Status.Good
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
  
    Display.Execution.Status(Execution.Status)
  --}}}  
:
--}}}  
--{{{  Request Sense
PROC Perform.Request.Sense.6([]BYTE SCSI.Command,
                             BYTE   Bus.Connection.State,
                             BYTE   Initiator.ID,
                             []BYTE Sense.Data,
                             BYTE   SCSI.Status,
                             INT16  Execution.Status)

  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  VAL BYTE Allocation.Length RETYPES SCSI.Command[4]:
  VAL BYTE Control.Byte RETYPES SCSI.Command[5]:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    Display.Execution.Status(Execution.Status)
    --}}}  
    --{{{  Data In Phase
    SCSI.T.Data.In.Phase ( TO.TARGET.CONTROLLER,
                           FROM.TARGET.CONTROLLER,
                           INT32 Allocation.Length,
                           Sense.Data,
                           Execution.Status)
    Display.Execution.Status(Execution.Status)
    --}}}  
    --{{{  Status & Message In
    --{{{  Prepare Message
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
    --}}}  
    --{{{  Send Status and Message
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    Display.Execution.Status(Execution.Status)
    SCSI.Status := BYTE SCSI.Status.Good
    --}}}  
    --}}}  
  --}}}  
:
--}}}  
--{{{  Write
PROC Perform.Write.6( []BYTE  SCSI.Command,
                      BYTE    Bus.Connection.State,
                      BYTE    Initiator.ID,
                      []BYTE  RAM.Disk,
                      VAL INT Block.Size,
                      []BYTE  Sense.Data,
                      BYTE    SCSI.Status,
                      INT16   Execution.Status)
  --{{{  Declarations
  VAL   BYTE Control.Byte RETYPES SCSI.Command[5]:
  INT   Transfer.Length:
  INT   Transfer.Count:
  INT32 Logical.Block.Address:
  INT   Start:
  [1]BYTE Message:
  BYTE Msg.Length:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    --}}}  
    --{{{  Decode Logical Block Address
    Logical.Block.Address := Zero.32
    []BYTE LBA.Bytes RETYPES Logical.Block.Address:
    SEQ
      LBA.Bytes[0] := SCSI.Command[3]
      LBA.Bytes[1] := SCSI.Command[2]
      LBA.Bytes[2] := BYTE ((INT SCSI.Command[1]) /\ #1F)
    --}}}  
    --{{{  Decode Transfer Count
    Transfer.Count := 0
    []BYTE TC.Bytes RETYPES Transfer.Count:
    SEQ
      TC.Bytes[0] := SCSI.Command[4]
    Transfer.Length := Transfer.Count * Block.Size
    --}}}  
    --{{{  Define Start position in memory
    Start := Block.Size * (INT Logical.Block.Address)
    --}}}  
    --{{{  Verify & Data Out if Ok
    IF
      ((Start + Transfer.Length) > (SIZE RAM.Disk))
        --Write will extend beyond end of media
        SEQ
          SCSI.Status := BYTE SCSI.Status.Check.Condition
          Sense.Data[2] := 5 (BYTE) --Illegal Request
      TRUE
        SEQ
          --{{{  Data Out Phase
          --{{{  Data Out Phase
          SCSI.T.Data.Out.Phase ( TO.TARGET.CONTROLLER,
                                  FROM.TARGET.CONTROLLER,
                                  INT32 Transfer.Length,
                                  [RAM.Disk FROM Start FOR Transfer.Length],
                                  Execution.Status)
          
          --Msg.Length := Zero.B
          
          --SCSI.Write.6 ( TO.DISK,
          --               FROM.DISK,
          --               Zero.B,
          --               Zero.B,
          --               Logical.Block.Address,
          --               Transfer.Count,
          --               Block.Size,
          --               Zero.B,
          --               Msg.Length,
          --               RAM.Disk,
          --               Message,
          --               SCSI.Status,
          --               Execution.Status)
          --}}}  
          Display.Execution.Status(Execution.Status)
          --}}}  
    --}}}  
    --{{{  Status & Message In
    --{{{  Status
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
    
    SCSI.Status := BYTE SCSI.Status.Good
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    
    Display.Execution.Status(Execution.Status)
    --}}}  
    --}}}  
  --}}}  
:
--}}}  
--{{{  Read
PROC Perform.Read.6(  []BYTE     SCSI.Command,
                      BYTE       Bus.Connection.State,
                      BYTE       Initiator.ID,
                      VAL []BYTE RAM.Disk,
                      VAL INT    Block.Size,
                      []BYTE     Sense.Data,
                      BYTE       SCSI.Status,
                      INT16      Execution.Status)
  --{{{  Declarations
  VAL   BYTE Control.Byte RETYPES SCSI.Command[5]:
  INT   Transfer.Length:
  INT   Transfer.Count:
  INT32 Logical.Block.Address:
  INT   Start:
  [1]BYTE Message:
  BYTE Msg.Length:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    --}}}  
    --{{{  Decode Logical Block Address
    Logical.Block.Address := Zero.32
    []BYTE LBA.Bytes RETYPES Logical.Block.Address:
    SEQ
      LBA.Bytes[0] := SCSI.Command[3]
      LBA.Bytes[1] := SCSI.Command[2]
      LBA.Bytes[2] := BYTE ((INT SCSI.Command[1]) /\ #1F)
    --}}}  
    --{{{  Decode Transfer Count
    Transfer.Count := 0
    []BYTE TC.Bytes RETYPES Transfer.Count:
    SEQ
      TC.Bytes[0] := SCSI.Command[4]
    Transfer.Length := Transfer.Count * Block.Size
    --}}}  
    --{{{  Define Start position in memory
    Start := Block.Size * (INT Logical.Block.Address)
    --}}}  
    --{{{  Verify & Data In if Ok
    IF
      ((Start + Transfer.Length) > (SIZE RAM.Disk))
        --Read will extend beyond end of media
        SEQ
          Sense.Data[2] := 5 (BYTE) --Illegal Request
          SCSI.Status := BYTE SCSI.Status.Check.Condition
      TRUE
        SEQ
          --{{{  Data In Phase
          Msg.Length := Zero.B
          
          --SCSI.Read.6 ( TO.DISK,
          --              FROM.DISK,
          --             Zero.B,
          --              Zero.B,
          --              Logical.Block.Address,
          --              Transfer.Count,
          --              Block.Size,
          --             Zero.B,
          --              RAM.Disk,
          --              Msg.Length,
          --              Message,
          --              SCSI.Status,
          --              Execution.Status)
          
          SCSI.T.Data.In.Phase  ( TO.TARGET.CONTROLLER,
                                  FROM.TARGET.CONTROLLER,
                                  INT32 Transfer.Length,
                                  [RAM.Disk FROM Start FOR Transfer.Length],
                                  Execution.Status)
          
          Display.Execution.Status(Execution.Status)
          --}}}  
    --}}}  
    --{{{  Status & Message In
    --{{{  Send Status and Message
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
    
    SCSI.Status := BYTE SCSI.Status.Good
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    
    Display.Execution.Status(Execution.Status)
    --}}}  
    --}}}  
  --}}}  
:
--}}}  
--{{{  Seek
PROC Perform.Seek.6( []BYTE     SCSI.Command,
                     BYTE       Bus.Connection.State,
                     BYTE       Initiator.ID,
                     VAL []BYTE RAM.Disk,
                     VAL INT    Block.Size,
                     []BYTE     Sense.Data,
                     BYTE       SCSI.Status,
                     INT16      Execution.Status)
  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  VAL BYTE Control.Byte RETYPES SCSI.Command[5]:
  INT32 Logical.Block.Address:
  --}}}  
  --{{{  Implementation
  SEQ
  
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
  
    --{{{  Decode Logical Block Address
    Logical.Block.Address := Zero.32
    []BYTE LBA.Bytes RETYPES Logical.Block.Address:
    SEQ
      LBA.Bytes[0] := SCSI.Command[3]
      LBA.Bytes[1] := SCSI.Command[2]
      LBA.Bytes[2] := BYTE ((INT SCSI.Command[1]) /\ #1F)
    --}}}  
  
    --{{{  Verify Start position in memory
    IF
      ((Block.Size *  (INT Logical.Block.Address)) >
      ((SIZE RAM.Disk) - Block.Size))
        SEQ
          Sense.Data[2] := 5 (BYTE) --Illegal Request
          SCSI.Status := BYTE SCSI.Status.Check.Condition
      TRUE
        SKIP
    --}}}  
  
    --{{{  Status
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
    
    SCSI.Status := BYTE SCSI.Status.Good
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    
    Display.Execution.Status(Execution.Status)
    --}}}  
  --}}}  
:
--}}}  
--}}}  
--{{{  10 Byte Commands
--{{{  Read Capacity.10
PROC Perform.Read.Capacity.10 ( []BYTE SCSI.Command,
                                BYTE   Bus.Connection.State,
                                BYTE   Initiator.ID,
                                []BYTE Read.Capacity.Data,
                                []BYTE Sense.Data,
                                BYTE   SCSI.Status,
                                INT16  Execution.Status)

  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  VAL BYTE Control.Byte RETYPES SCSI.Command[9]:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    --}}}  
    Sense.Data[2] := 0 (BYTE)
    --{{{  Data In Phase
    SCSI.T.Data.In.Phase ( TO.TARGET.CONTROLLER,
                           FROM.TARGET.CONTROLLER,
                           8 (INT32),
                           Read.Capacity.Data,
                           Execution.Status)
    Display.Execution.Status(Execution.Status)
    --}}}  
    --{{{  Status & Message In
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
    
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    
    Display.Execution.Status(Execution.Status)
    --}}}  
  --}}}  
:
--}}}  
--{{{  Read.10
PROC Perform.Read.10( []BYTE     SCSI.Command,
                      BYTE       Bus.Connection.State,
                      BYTE       Initiator.ID,
                      VAL []BYTE RAM.Disk,
                      VAL INT    Block.Size,
                      []BYTE     Sense.Data,
                      BYTE       SCSI.Status,
                      INT16      Execution.Status)
  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  VAL BYTE Control.Byte RETYPES SCSI.Command[9]:
  INT Transfer.Count:
  INT Transfer.Length:
  INT Start:
  INT32 Logical.Block.Address:
  --}}}  
  --{{{  Implementation
  SEQ
    --DEBUG.MESSAGE("Read 10")
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    --}}}  
    --{{{  Decode Logical Block Address
    []BYTE LBA.Bytes RETYPES Logical.Block.Address:
    SEQ
      LBA.Bytes[0] := SCSI.Command[5]
      LBA.Bytes[1] := SCSI.Command[4]
      LBA.Bytes[2] := SCSI.Command[3]
      LBA.Bytes[3] := SCSI.Command[2]
    --}}}  
    --{{{  Decode Transfer Count
    Transfer.Count := 0
    []BYTE TC.Bytes RETYPES Transfer.Count:
    SEQ
      TC.Bytes[0] := SCSI.Command[8]
      TC.Bytes[1] := SCSI.Command[7]
    Transfer.Length :=  Transfer.Count * Block.Size
    --}}}  
    --{{{  Define Start position in memory
    Start := Block.Size * (INT Logical.Block.Address)
    --}}}  
    --{{{  Verify & Data In if Ok
    IF
      ((Start + Transfer.Length) > (SIZE RAM.Disk))
        --Read will extend beyond end of media
        SEQ
          Sense.Data[2] := 5 (BYTE) --Illegal Request
          SCSI.Status := BYTE SCSI.Status.Check.Condition
      TRUE
        SEQ
          SCSI.Status := BYTE SCSI.Status.Good
          --{{{  Data In Phase
          --{{{  
          --SCSI.T.Data.In.Phase  ( TO.TARGET.CONTROLLER,
          --                        FROM.TARGET.CONTROLLER,
          --                        INT32 Transfer.Length,
          --                        [RAM.Disk FROM Start FOR Transfer.Length],
          --                        Execution.Status)
          --}}}  
          --{{{  
          SCSI.T.Data.In.Phase.DL  ( TO.TARGET.CONTROLLER,
                                     DATA.TO.TARGET.CONTROLLER,
                                     FROM.TARGET.CONTROLLER,
                                     DATA.FROM.TARGET.CONTROLLER,
                                     INT32 Transfer.Length,
                                     interval.offset,
                                     [RAM.Disk FROM Start FOR Transfer.Length],
                                     Execution.Status)
          --}}}  
          Display.Execution.Status(Execution.Status)
          --}}}  
    --}}}  
    --{{{  Status & Message In
    --{{{  Prepare Message
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
    --}}}  
    --{{{  Send Status and Message
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    Display.Execution.Status(Execution.Status)
    --}}}  
    --}}}  
  --}}}  
:
--}}}  
--{{{  Write.10
PROC Perform.Write.10( []BYTE  SCSI.Command,
                       BYTE    Bus.Connection.State,
                       BYTE    Initiator.ID,
                       []BYTE  RAM.Disk,
                       VAL INT Block.Size,
                       []BYTE  Sense.Data,
                       BYTE    SCSI.Status,
                       INT16   Execution.Status)
  --{{{  Declarations
  VAL BYTE Control.Byte RETYPES SCSI.Command[9]:
  [1]BYTE Message:
  BYTE Msg.Length:
  INT Transfer.Length:
  INT Transfer.Count:
  INT Start:
  INT32 Logical.Block.Address:
  --}}}  
  --{{{  Implementation
  SEQ
    --DEBUG.MESSAGE("Write 10")
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    --}}}  
    --{{{  Decode Logical Block Address
    []BYTE LBA.Bytes RETYPES Logical.Block.Address:
    SEQ
      LBA.Bytes[0] := SCSI.Command[5]
      LBA.Bytes[1] := SCSI.Command[4]
      LBA.Bytes[2] := SCSI.Command[3]
      LBA.Bytes[3] := SCSI.Command[2]
    --}}}  
    --{{{  Decode Transfer Count
    Transfer.Count := 0
    []BYTE TC.Bytes RETYPES Transfer.Count:
    SEQ
      TC.Bytes[0] := SCSI.Command[8]
      TC.Bytes[1] := SCSI.Command[7]
    Transfer.Length :=  Transfer.Count * Block.Size
    --}}}  
    --{{{  Define Start position in memory
    Start := Block.Size * (INT Logical.Block.Address)
    --}}}  
    --{{{  Verify & Data Out if Ok
    IF
      ((Start + Transfer.Length)  > (SIZE RAM.Disk))
        --Write will extend beyond end of media
        SEQ
          Sense.Data[2] := 5 (BYTE) --Illegal Request
          SCSI.Status := BYTE SCSI.Status.Check.Condition
      TRUE
        SEQ
          SCSI.Status :=  BYTE SCSI.Status.Good
          --{{{  Data Out Phase
          --{{{  
          --SCSI.T.Data.Out.Phase.SL ( TO.TARGET.CONTROLLER,
          --                           FROM.TARGET.CONTROLLER,
          --                           INT32 Transfer.Length,
          --                           [RAM.Disk FROM Start FOR Transfer.Length],
          --                           Execution.Status)
          --}}}  
          SCSI.T.Data.Out.Phase.DL ( TO.TARGET.CONTROLLER,
                                     DATA.TO.TARGET.CONTROLLER,
                                     FROM.TARGET.CONTROLLER,
                                     DATA.FROM.TARGET.CONTROLLER,
                                     INT32 Transfer.Length,
                                     interval.offset,
                                     [RAM.Disk FROM Start FOR Transfer.Length],
                                     Execution.Status)
          Display.Execution.Status(Execution.Status)
          --}}}  
    --}}}  
    --{{{  Status & Message In
    --{{{  Message in
    
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
    --}}}  
    --{{{  Status
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    Display.Execution.Status(Execution.Status)
    --}}}  
    --}}}  
  --}}}  
:
--}}}  
--{{{  Seek.10
PROC Perform.Seek.10 ( []BYTE     SCSI.Command,
                       BYTE       Bus.Connection.State,
                       BYTE       Initiator.ID,
                       VAL []BYTE RAM.Disk,
                       VAL INT    Block.Size,
                       []BYTE     Sense.Data,
                       BYTE       SCSI.Status,
                       INT16      Execution.Status)
  --{{{  Declarations
  [1]BYTE Message:
  BYTE Msg.Length:
  INT32 Logical.Block.Address:
  VAL BYTE Control.Byte RETYPES SCSI.Command[9]:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Reconnect if neccessary
    Reconnect(Bus.Connection.State, Initiator.ID, Execution.Status)
    --}}}  
  
    --{{{  Decode Logical Block Address
    []BYTE LBA.Bytes RETYPES Logical.Block.Address:
    SEQ
      LBA.Bytes[0] := SCSI.Command[5]
      LBA.Bytes[1] := SCSI.Command[4]
      LBA.Bytes[2] := SCSI.Command[3]
      LBA.Bytes[3] := SCSI.Command[2]
    --}}}  
  
    --{{{  Verify Start position in memory
    IF
      ((Block.Size *  (INT Logical.Block.Address)) >
      ((SIZE RAM.Disk) - Block.Size))
        SEQ
          Sense.Data[2] := 5 (BYTE) --Illegal Request
          SCSI.Status := BYTE SCSI.Status.Check.Condition
      TRUE
        SKIP
    --}}}  
  
    Msg.Length := 1 (BYTE)
    IF
      (Control.Byte = SCSI.Non.Linked.Command)
        Message[0] := BYTE Command.Complete
      TRUE
        Message[0] := BYTE Terminate.IO.Process
  
    SCSI.T.Status.And.Message.In ( TO.TARGET.CONTROLLER,
                                   FROM.TARGET.CONTROLLER,
                                   SCSI.Status,
                                   Control.Byte,
                                   Msg.Length,
                                   Message,
                                   Execution.Status)
    Display.Execution.Status(Execution.Status)
  
  --}}}  
:
--}}}  
--}}}  
