--{{{  p redraw screen
PROC p.redraw.screen (CHAN OF SP fs, ts, VAL []BYTE data, title,
       INT start.address, address, cur.line, no.lines, cur.col, part.line.cols,
       BOOL first.page, last.page, part.line, VAL BOOL read.write, hex)
  INT full.lines:
  VAL bytes.on.screen IS screen.lines * no.cols:
  SEQ
    --{{{  correct address
    IF
      address < 0
        address := 0
      address >= (SIZE data)
        address := (SIZE data) - 1
      TRUE
        SKIP
    --}}}
    --{{{  correct start.address
    SEQ
      VAL min.start.address IS (address - bytes.on.screen) + 1:
      IF
        start.address > address
          start.address := address
        start.address < min.start.address
          start.address := min.start.address
        start.address < 0
          start.address := 0
        TRUE
          SKIP
      start.address := (start.address / 16) * 16
    --}}}
    --{{{  calculate state variables
    SEQ
      --{{{  page layout
      INT bytes.to.end:
      SEQ
        bytes.to.end := (SIZE data) - start.address
      
        full.lines   := bytes.to.end / no.cols
        part.line.cols   := bytes.to.end \ no.cols
        VAL part.line.empty IS part.line.cols = 0:
        IF
          screen.lines <= full.lines
            SEQ
              full.lines := screen.lines
              no.lines := full.lines
              part.line := FALSE
          part.line.empty
            SEQ
              no.lines := full.lines
              part.line := FALSE
          TRUE
            SEQ
              no.lines := full.lines + 1
              part.line := TRUE
      --}}}
      --{{{  first and last page
      SEQ
        first.page := (start.address <= 0)
        VAL max.address IS start.address + (screen.lines * no.cols):
        VAL last.data   IS (SIZE data) - 1:
        last.page := (max.address >= last.data)
      --}}}
      --{{{  cursor position
      SEQ
        cur.line := (address - start.address) / no.cols
        cur.col := (address - start.address) \ no.cols
      --}}}
    
    --}}}
    --{{{  redraw screen
    SEQ
      screen.clear (fs, ts)
      screen.title (fs, ts, 0, title)
      VAL help.line IS
        "Esc=quit: F1=Help: F2=ascii/hex: F3=write enable: F4=goto address":
      screen.title (fs, ts, 1, help.line)
      screen.header (fs, ts, address,
                     read.write, first.page, last.page, cur.line, cur.col)
      screen.refresh (fs, ts, data, start.address, cur.line, full.lines,
                    cur.col, part.line.cols, part.line, hex)
    --}}}
:
--}}}

--{{{  p page up
PROC p.page.up (CHAN OF SP fs, ts, VAL []BYTE data, title,
             INT start.address, address, line, no.lines, col, part.line.cols,
             BOOL first.page, last.page, part.line, VAL BOOL read.write, hex)
  INT min.address:
  IF
    first.page
      SKIP
    TRUE
      SEQ
        --{{{  calculate new start address
        
        VAL max.lines.up IS screen.lines - overlap:
        SEQ
          min.address := start.address - (max.lines.up * no.cols)
          IF
            min.address < 0
              start.address := 0
            TRUE
              start.address := min.address
        --}}}
        --{{{  find new cursor position
        
        VAL end.address IS (start.address + (screen.lines * no.cols)) - 1:
        IF
          address > end.address
            --{{{  move to bottom screen line
            SEQ
              line := screen.lines
              address := ((end.address - no.cols) + 1) + col
            --}}}
          TRUE
            --{{{  find new location on screen
            SEQ
              line := (address - start.address) / no.cols
              col := (address - start.address) \ no.cols
            --}}}
        --}}}
        p.redraw.screen (fs, ts, data, title,
                  start.address, address, line, no.lines, col, part.line.cols,
                  first.page, last.page, part.line, read.write, hex)
:
--}}}
--{{{  p page down
PROC p.page.down (CHAN OF SP fs, ts, VAL []BYTE data, title,
             INT start.address, address, line, no.lines, col, part.line.cols,
             BOOL first.page, last.page, part.line, VAL BOOL read.write, hex)
  IF
    last.page
      SKIP
    TRUE
      SEQ
        --{{{  calculate new start address etc
        BOOL off.the.end:
        INT end.address:
        VAL max.lines.down IS screen.lines - overlap:
        VAL max.address.inc IS max.lines.down * no.cols:
        SEQ
          VAL old.end.address IS (start.address-1) + (screen.lines * no.cols):
          end.address := old.end.address + max.address.inc
          off.the.end := end.address >= (SIZE data)
          IF
            off.the.end
              VAL max.address IS (SIZE data) - 1:
              SEQ
                end.address := (((max.address) / no.cols) + 1) * no.cols
                start.address := end.address - (screen.lines * no.cols)
            TRUE
              start.address := start.address + max.address.inc
        --}}}
        --{{{  find new cursor position
        IF
          address < start.address
            --{{{  move to top screen line
            SEQ
              line := 0
              address := start.address + col
            --}}}
          TRUE
            --{{{  find new location on screen
            SEQ
              line := (address - start.address) / no.cols
              col := (address - start.address) \ no.cols
            --}}}
        --}}}
        p.redraw.screen (fs, ts, data, title,
                  start.address, address, line, no.lines, col, part.line.cols,
                  first.page, last.page, part.line, read.write, hex)
:
--}}}

--{{{  p cursor up
PROC p.cursor.up (CHAN OF SP fs, ts, VAL []BYTE data, title,
              INT start.address, address, line, no.lines, col, part.line.cols,
              BOOL first.page, last.page, part.line, VAL BOOL read.write, hex)
  SEQ
    IF
      (line <= 0) AND first.page
        SKIP
      line <= 0
        SEQ
          p.page.up (fs, ts, data, title,
               start.address, address, line, no.lines, col, part.line.cols,
               first.page, last.page, part.line, read.write, hex)
          screen.cursor.up (fs, ts)
          line := line - 1
          address := address - no.cols
      TRUE
        SEQ
          screen.cursor.up (fs, ts)
          line := line - 1
          address := address - no.cols
:
--}}}
--{{{  p cursor down
PROC p.cursor.down (CHAN OF SP fs, ts, VAL []BYTE data, title,
               INT start.address, address, line, no.lines, col, part.line.cols,
               BOOL first.page, last.page, part.line, VAL BOOL read.write, hex)
  BOOL last.line:
  SEQ
    --{{{  deduce whether last line
    last.line := (line >= (no.lines - 1))
    VAL beyond.part.line IS part.line AND (col >= part.line.cols):
    VAL next.last.line   IS (line >= (no.lines - 2)):
    last.line := last.line OR (next.last.line AND beyond.part.line)
    --}}}
    IF
      last.line AND last.page
        --{{{  do nothing
        SKIP
        --}}}
      last.line     -- not last page
        --{{{  page down and cursor down
        SEQ
          p.page.down (fs, ts, data, title,
               start.address, address, line, no.lines, col, part.line.cols,
               first.page, last.page, part.line, read.write, hex)
          screen.cursor.down (fs, ts)
          line := line + 1
          address := address + no.cols
        --}}}
      TRUE
        --{{{  cursor down
        SEQ
          screen.cursor.down (fs, ts)
          line := line + 1
          address := address + no.cols
        --}}}
:
--}}}
--{{{  p cursor left
PROC p.cursor.left (CHAN OF SP fs, ts, VAL []BYTE data, title,
              INT start.address, address, line, no.lines, col, part.line.cols,
              BOOL first.page, last.page, part.line, VAL BOOL read.write, hex)
  BOOL start.of.line, start.of.page, start.of.data:
  SEQ
    --{{{  calculate booleans
    start.of.line := (col <= 0)
    VAL first.line IS (line = 0):
    start.of.page := start.of.line AND first.line
    start.of.data := start.of.page AND first.page
    --}}}
    IF
      start.of.data
        SKIP
      start.of.page
        --{{{  page up and next line up
        SEQ
          p.page.up (fs, ts, data, title,
                   start.address, address, line, no.lines, col, part.line.cols,
                   first.page, last.page, part.line, read.write, hex)
          --{{{  next line up
          SEQ
            line := line - 1
            col := no.cols - 1
            screen.goto (fs, ts, line, col, hex)
            address := address - 1
          --}}}
        --}}}
      start.of.line
        --{{{  next line up
        SEQ
          line := line - 1
          col := no.cols - 1
          screen.goto (fs, ts, line, col, hex)
          address := address - 1
        --}}}
      TRUE
        --{{{  cursor left
        
        INT width:
        SEQ
          --{{{  calculate width
          IF
            hex
              width := col.width
            TRUE
              width := 1
          --}}}
          screen.cursor.left (fs, ts, width)
          col := col - 1
          address := address - 1
        --}}}

:
--}}}
--{{{  p cursor right
PROC p.cursor.right (CHAN OF SP fs, ts, VAL []BYTE data, title,
                    INT start.address, address, line, no.lines, col, part.line.cols,
                    BOOL first.page, last.page, part.line,
                    VAL BOOL read.write, hex)
  BOOL end.of.data, end.of.page, full.line.end:
  SEQ
    --{{{  calculate booleans
    SEQ
      full.line.end := (col >= (no.cols - 1))
      VAL last.line     IS line >= (no.lines - 1):
      VAL on.part.line  IS last.line AND part.line:
      VAL part.line.end IS on.part.line AND (col >= (part.line.cols - 1)):
      VAL end.of.line   IS part.line.end OR full.line.end:
      end.of.page  := last.line AND end.of.line
      end.of.data  := end.of.page AND last.page
    --}}}
    IF
      end.of.data
        SKIP
      end.of.page
        --{{{  page down and next line down
        SEQ
          p.page.down (fs, ts, data, title,
                       start.address, address, line, no.lines, col, part.line.cols,
                       first.page, last.page, part.line, read.write, hex)
          --{{{  next line down
          SEQ
            line := line + 1
            col := 0
            screen.goto (fs, ts, line, col, hex)
            address := address + 1
          --}}}
        --}}}
      full.line.end
        --{{{  next line down
        SEQ
          line := line + 1
          col := 0
          screen.goto (fs, ts, line, col, hex)
          address := address + 1
        --}}}
      TRUE
        --{{{  cursor.right
        INT width:
        SEQ
          --{{{  calculate width
          IF
            hex
              width := col.width
            TRUE
              width := 1
          --}}}
          screen.cursor.right (fs, ts, width)
          col := col + 1
          address := address + 1
        --}}}

:
--}}}

--{{{  p overwrite hex
PROC p.overwrite.hex (CHAN OF SP fs, ts, []BYTE data, VAL []BYTE title,
              INT start.address, address, line, no.lines, col, part.line.cols,
              BOOL first.page, last.page, part.line,
              VAL BOOL read.write, VAL INT key1)
  #USE "hostio.lib"
  --{{{  declarations
  VAL hex IS TRUE:
  --{{{  FUNCTION hex value
  INT FUNCTION hex.value (VAL INT ch)
    INT value:
    VALOF
      SEQ
        value := char.pos (BYTE ch, "0123456789ABCDEF")
        IF
          value < 0
            value := char.pos (BYTE ch, "abcdef") + 10
          TRUE
            SKIP
      RESULT value
  :
  --}}}
  INT value, key2:
  BOOL key.invalid, special, ok:
  --}}}
  SEQ
    --{{{  deal with first digit
    so.write.string (fs, ts, [BYTE key1, '*s'])
    screen.backspace (fs, ts)
    --}}}
    key.invalid := TRUE
    WHILE key.invalid
      SEQ
        read.key (fs, ts, key2, special, ok)
        IF
          NOT ok
            SKIP
          special AND (key2 = k.delete)
            --{{{  restore previous state and abort
            SEQ
              screen.goto (fs, ts, line, col, hex)
              screen.write.hex.byte (fs, ts, data[address])
              screen.goto (fs, ts, line, col, hex)
              key.invalid := FALSE
            --}}}
          (NOT special) AND is.hex.digit (BYTE key2)
            --{{{  overwrite data, screen hex and ascii
            
            SEQ
              value := (hex.value (key1) * 16) + hex.value (key2)
              data[address] := BYTE value
              so.write.char (fs, ts, BYTE key2)
            
              screen.change.ascii (fs, ts, line, col, BYTE value)
              p.cursor.right (fs, ts, data, title,
                    start.address, address, line, no.lines, col, part.line.cols,
                    first.page, last.page, part.line, read.write, hex)
              key.invalid := FALSE
            --}}}
          TRUE
            SKIP

:
--}}}
--{{{  p overwrite ascii
PROC p.overwrite.ascii (CHAN OF SP fs, ts, []BYTE data, VAL []BYTE title,
              INT start.address, address, line, no.lines, col, part.line.cols,
              BOOL first.page, last.page, part.line,
              VAL BOOL read.write, VAL BYTE key)

  #USE "hostio.lib"
  SEQ
    screen.change.hex (fs, ts, line, col, key)
    so.write.char (fs, ts, key)
    data[address] := key
    p.cursor.right (fs, ts, data, title,
        start.address, address, line, no.lines, col, part.line.cols,
        first.page, last.page, part.line, read.write, FALSE)
:
--}}}

--{{{  p goto address
PROC p.goto.address (CHAN OF SP fs, ts, VAL []BYTE data, title,
             INT start.address, address, line, no.lines, col, part.line.cols,
             BOOL first.page, last.page, part.line, VAL BOOL read.write, hex)
  #USE "hostio.lib"
  BOOL error:
  INT n:
  SEQ
    screen.prompt (fs, ts, "Go to what address? ")
    so.read.echo.any.int (fs, ts, n, error)
    IF
      error
        SKIP
      TRUE
        SEQ
          --{{{  correct address
          VAL last IS (SIZE data) - 1:
          IF
            n < 0
              address := 0
            n > last
              address := last
            TRUE
              address := n
          --}}}
          --{{{  redraw screen
          VAL bytes.on.screen IS screen.lines * no.cols:
          start.address := address - (bytes.on.screen / 2)
          p.redraw.screen (fs, ts, data, title,
                  start.address, address, line, no.lines, col, part.line.cols,
                  first.page, last.page, part.line, read.write, hex)
          --}}}

:
--}}}

--{{{  p insert
PROC p.insert (CHAN OF SP fs, ts, []BYTE data, VAL []BYTE title,
             INT start.address, address, line, no.lines, col, part.line.cols,
             BOOL first.page, last.page, part.line, VAL BOOL hex)
  --{{{  block insert
  PROC p.block.insert ([]BYTE data, VAL INT address)
    VAL bytes.to.move IS ((SIZE data) - address) - 1:
    SEQ
      SEQ i = 1 FOR bytes.to.move
        VAL byte IS (SIZE data) - i:
        data[byte] := data[byte-1]
      data[address] := 0(BYTE)
  :
  --}}}
  BOOL special, ok:
  INT key:
  SEQ
    screen.prompt (fs, ts, "Warning: Last byte of array will be lost. Repeat to proceed.")
    read.key (fs, ts, key, special, ok)
    IF
      (ok AND special) AND (key = k.insert)
        SEQ
          screen.prompt (fs, ts, "Moving array...")
          p.block.insert (data, address)
      TRUE
        SKIP
    p.redraw.screen (fs, ts, data, title,
            start.address, address, line, no.lines, col, part.line.cols,
            first.page, last.page, part.line, TRUE, hex)
:
--}}}
--{{{  p delete
PROC p.delete (CHAN OF SP fs, ts, []BYTE data, VAL []BYTE title,
             INT start.address, address, line, no.lines, col, part.line.cols,
             BOOL first.page, last.page, part.line, VAL BOOL hex)
  --{{{  block delete
  PROC p.block.delete ([]BYTE data, VAL INT address)
    VAL last          IS (SIZE data) - 1:
    VAL bytes.to.move IS last - address:
    SEQ
      SEQ i = address FOR bytes.to.move
        data[i] := data[i+1]
      data[last] := 0(BYTE)
  :
  --}}}
  BOOL special, ok:
  INT key:
  SEQ
    screen.prompt (fs, ts, "Warning: Current byte of array will be lost. Repeat to proceed.")
    read.key (fs, ts, key, special, ok)
    IF
      (ok AND special) AND (key = k.delete)
        SEQ
          screen.prompt (fs, ts, "Moving array...")
          p.block.delete (data, address)
      TRUE
        SKIP
    p.redraw.screen (fs, ts, data, title,
            start.address, address, line, no.lines, col, part.line.cols,
            first.page, last.page, part.line, TRUE, hex)
:
--}}}
