--{{{  Header
--------------------------------------------------------------------------------
--
-- Security    : Copyright 1990, 1991 INMOS Limited
--
-- Author      : Paul Fellows
--
-- Date        : 19-Jan-1990
--
-- File Name   : DEFPARAM.INC
--
-- Project     : F002C - SCSI Tram Software
--
-- Version     : 0.1
--
-- Division    : IQ Systems
--
-- Limitations : This software may be distributed in source form.
--
-- Description : Hardware definitions for B422 SCSI TRAM
--
--------------------------------------------------------------------------------
-- Schedule of Amendments
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--}}}  
PROC Define.Parameters(CHAN OF SP fs, ts,
                       CHAN OF TO.SCSI tscsi,
                       CHAN OF FROM.SCSI fscsi)
  BOOL Going:
  INT  reply:
  SEQ
    Going := TRUE
    Execution.Status := SCSI.E.Good
    Clear.Screen(fs, ts)
    WHILE Going
      SEQ
        --{{{  Prompt User
        Display.Options(fs, ts,
                        Main.Title,
                        "Define Parameters",
                        ["Exit               ",
                         "Parity             ",
                         "Data Transfer Mode ",
                         "Disconnection      ",
                         "ATN                "],
                         Status.Message,
                         reply)
        --}}}  
        --{{{  Do User request
        CASE reply
          0
            Going := FALSE
          1
            --{{{  Prompt Operator for Parity Info
            INT reply:
            SEQ
              --{{{  Parity Checking
              so.ask(fs, ts,
                     "Support Parity Checking > ",
                     "YN",
                     TRUE,
                     TRUE,
                     reply)
              
              CASE reply
                0
                  Parity.Checking := Enable.Parity.Checking
                1
                  Parity.Checking := Disable.Parity.Checking
                ELSE
                  so.write.string.nl(fs, ts, "No Change Made")
              
              so.write.nl(fs, ts)
              --}}}  
              --{{{  Parity Generation
              INT reply:
              SEQ
                so.ask(fs, ts,
                       "Support Parity Generation   > ",
                       "YN",
                       TRUE,
                       TRUE,
                       reply)
                CASE reply
                  0
                    Parity.Generation := Enable.Parity.Generation
                  1
                    Parity.Generation := Disable.Parity.Generation
                  ELSE
                    so.write.string.nl(fs, ts, "No Change Made")
                so.write.nl(fs, ts)
              --}}}  
              --{{{  Update Driver
              SCSI.Initialise ( tscsi,
                                fscsi,
                                Data.Phase.Time.Out,
                                Interrupt.Time.Out,
                                Parity.Generation,
                                Parity.Checking,
                                Cable.Mode,
                                Do.Not.Reset.SCSI.Controller,
                                Do.Not.Reset.SCSI.Bus,
                                Do.Not.Self.Test,
                                Execution.Status)
              Prepare.Status.Message(Execution.Status, Status.Message)
              --}}}  
            --}}}  
          2
            --{{{  Data Transfer Mode
            Data.Transfer.Mode.Selection(fs, ts, tscsi, fscsi)
            --}}}  
          3
            --{{{  Promt Operator for Disconnection Mode
            INT reply:
            SEQ
              so.ask(fs, ts,
                     "Support Disconnection ? ",
                     "YN",
                     TRUE,
                     TRUE,
                     reply)
            
              CASE reply
                0
                  Disconnect.Mode := Disconnection.Disabled
                1
                  Disconnect.Mode := Disconnection.Enabled
                ELSE
                  Not.Implemented(fs, ts)
            
              so.write.nl(fs, ts)
            --}}}  
          4
            --{{{  Prompt Operator for Select With ATN
            INT reply:
            SEQ
              so.ask(fs, ts,
                     "Select With ATN ? ",
                     "NY",
                     TRUE,
                     TRUE,
                     ATN)
            
              CASE ATN
                0
                  SCSI.Select.Mode ( tscsi,
                                     fscsi,
                                     0 (BYTE),    -- Select Without Attention
                                     Execution.Status)
                1
                  SCSI.Select.Mode ( tscsi,
                                     fscsi,
                                     1 (BYTE),    -- Select With Attention
                                     Execution.Status)
                ELSE
                  Not.Implemented(fs, ts)
              Prepare.Status.Message(Execution.Status, Status.Message)
            --}}}  
          ELSE
            so.write.string.nl(fs, ts, "Not Implemented")
        --}}}  
:

