#COMMENT  "(c) Copyright INMOS Limited 1990, 1991"
#COMMENT  "F002C Product Release SCSI software libraries"
--{{{  Header
--------------------------------------------------------------------------------
--
-- Security    : Copyright 1990 INMOS Limited.
--
-- Author      :
--
-- Date        :
--
-- File Name   : .OCC
--
-- Project     : F002 - SCSI Tram Software
--
-- Module No.  : F002.009.OCC
--
-- Version     :
--
-- Division    : IQ Systems
--
-- Limitations : This software may NOT be distributed in source form.
--
-- Description : Hex Editor
--
--------------------------------------------------------------------------------
-- Schedule of Amendments
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--}}}
--{{{  Includes
#INCLUDE "hostio.inc"
#INCLUDE "hexedhdr.inc"
--}}}
--{{{  Implementation
PROC Hex.Editor( CHAN OF SP fs, ts,
                 VAL INT t.address,
                 []BYTE data, VAL []BYTE title)
  --{{{  Uses
  #USE "hostio.lib"
  #USE "string.lib"
  #USE "debug.lib"
  --}}}
  --{{{  Includes
  #INCLUDE "utility.inc"
  #INCLUDE "term.inc"
  #INCLUDE "disp.inc"
  #INCLUDE "cont.inc"
  #INCLUDE "locproc.inc"
  --}}}
  --{{{  read iterm
  PROC read.iterm ()
    SKIP
  :
  --}}}
  --{{{  declarations
  INT key, line, start.address, address, col, no.lines, part.line.cols:
  BOOL hex, read.write, going, last.page, first.page, part.line, special, ok:
  --}}}
  SEQ
    --{{{  initialise
    address := t.address
    start.address := address
    hex := TRUE
    screen.clear (fs, ts)
    read.iterm ()
    p.redraw.screen (fs, ts, data, title,
                 start.address, address, line, no.lines, col, part.line.cols,
                  first.page, last.page, part.line, read.write, hex)
    going := TRUE
    --}}}
    WHILE going
      SEQ
        --{{{  display monitor information
        SEQ
          screen.header (fs, ts, address,
                         read.write, first.page, last.page, line, col)
          screen.goto (fs, ts, line, col, hex)
        
        --}}}
        read.key (fs, ts, key, special, ok)
        IF
          NOT ok   -- key not read
            SKIP
          special
            --{{{  special key
            IF
              key = k.quit
                going := FALSE
              key = k.up
                --{{{  cursor up
                p.cursor.up (fs, ts, data, title,
                      start.address, address, line, no.lines, col, part.line.cols,
                      first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.down
                --{{{  cursor down
                p.cursor.down (fs, ts, data, title,
                      start.address, address, line, no.lines, col, part.line.cols,
                      first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.left
                --{{{  move cursor left
                p.cursor.left (fs, ts, data, title,
                       start.address, address, line, no.lines, col, part.line.cols,
                       first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.right
                --{{{  move cursor right
                p.cursor.right (fs, ts, data, title,
                       start.address, address, line, no.lines, col, part.line.cols,
                       first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.pgup
                --{{{  page up
                p.page.up (fs, ts, data, title,
                      start.address, address, line, no.lines, col, part.line.cols,
                      first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.pgdn
                --{{{  page down
                p.page.down (fs, ts, data, title,
                      start.address, address, line, no.lines, col, part.line.cols,
                      first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.insert
                --{{{  insert
                IF
                  read.write
                    p.insert (fs, ts, data, title,
                          start.address, address, line, no.lines, col, part.line.cols,
                          first.page, last.page, part.line, hex)
                  TRUE
                    SKIP
                --}}}
              key = k.delete
                --{{{  delete
                IF
                  read.write
                    p.delete (fs, ts, data, title,
                          start.address, address, line, no.lines, col, part.line.cols,
                          first.page, last.page, part.line, hex)
                  TRUE
                    SKIP
                --}}}
              key = k.goto
                --{{{  goto address
                p.goto.address (fs, ts, data, title,
                      start.address, address, line, no.lines, col, part.line.cols,
                      first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.top
                --{{{  top of array
                SEQ
                  address := 0
                  start.address := 0
                  p.redraw.screen (fs, ts, data, title,
                      start.address, address, line, no.lines, col, part.line.cols,
                      first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.bottom
                --{{{  bottom of array
                SEQ
                  address := (SIZE data) - 1
                  VAL min.bytes.on.screen IS (screen.lines - 1) * no.cols:
                  start.address := address - min.bytes.on.screen
                  p.redraw.screen (fs, ts, data, title,
                      start.address, address, line, no.lines, col, part.line.cols,
                      first.page, last.page, part.line, read.write, hex)
                --}}}
              key = k.rw.toggle
                read.write := NOT read.write
              key = k.ah.toggle
                hex := NOT hex
              key = k.clear.data
                --{{{  clear data
                SEQ
                  address := 0
                  start.address := 0
                  initBYTEvec(data, Zero.B, SIZE data)
                  p.redraw.screen (fs, ts, data, title,
                      start.address, address, line, no.lines, col, part.line.cols,
                      first.page, last.page, part.line, read.write, hex)
                --}}}
              -- other keys not used should be skipped or they will be inserted
              TRUE                -- invalid key
                SKIP
            --}}}
          (read.write AND hex) AND is.hex.digit (BYTE key)
            --{{{  overwrite hex
            p.overwrite.hex (fs, ts, data, title,
                        start.address, address, line, no.lines, col, part.line.cols,
                        first.page, last.page, part.line, read.write, key)
            --}}}
          read.write AND (NOT hex)
            --{{{  overwrite ascii
            p.overwrite.ascii (fs, ts, data, title,
                        start.address, address, line, no.lines, col, part.line.cols,
                        first.page, last.page, part.line, read.write, BYTE key)
            --}}}
          TRUE                -- invalid key
            SKIP
    screen.clear (fs, ts)
:
--}}}
