PROC Generic.SCSI.Command(CHAN OF SP fs, ts,
                          CHAN OF TO.SCSI tscsi,
                          CHAN OF FROM.SCSI fscsi)
  --{{{  Declarations
  INT32 Allocation.Length:
  BYTE Msg.Length:
  BYTE SCSI.Status:
  BYTE Direction:
  BOOL Success:
  INT  Command.Length:
  INT  Data.Transfer.Length:
  [2048]BYTE SCSI.Command:
  INT16 Execution.Status:
  [1]BYTE Message:
  BOOL Escape:
  --}}}
  SEQ
    --{{{  Preamble
    Escape := FALSE
    Msg.Length := Zero.B
    Clear.Screen(fs, ts)
    Display.Line(fs, ts, "Generic SCSI Command Interface")
    so.write.nl(fs, ts)
    so.write.string.nl(fs, ts, "Length of SCSI Command [ 6 | 10 | 12 ] ?")
    Success := FALSE
    WHILE NOT Success
      SEQ
        so.read.echo.int(fs, ts, Command.Length, Success)
        IF
          ((Command.Length < 0) OR (Command.Length > 12))
            Success := FALSE
          TRUE
            SKIP
    
    SEQ i = 0 FOR SIZE SCSI.Command
      SCSI.Command[i] := Zero.B
    --{{{  Edit SCSI Command
    Hex.Editor(fs, ts, 0,
               SCSI.Command,
               "Edit SCSI Command")
    --}}}
    --{{{  Direction Choice
    INT reply:
    SEQ
      WHILE reply <> 0
        SEQ
          Display.Options(fs, ts,
                          Main.Title,
                          "Data Transfer Direction",
                          ["Exit            ",
                           "No Data Transfer",
                           "Data Out phase  ",
                           "Data In phase   "],
                           Status.Message,
                           reply)
          --{{{  perform user request
          CASE reply
            0
              Escape := TRUE
            1
              --{{{  Non Data Transfer
              [1]BYTE Null.Buff.1:
              [1]BYTE Null.Buff.2:
              SEQ
                Direction := Non.Data.Transfer
                --{{{  Transfer Command to SCSI Device
                SCSI.HADD.IF ( tscsi,
                               fscsi,
                               BYTE Target.ID,
                               BYTE LUN,
                               BYTE Command.Length,
                               SCSI.Command,
                               Direction,
                               Zero.32,
                               Null.Buff.1,
                               Zero.32,
                               Null.Buff.2,
                               Msg.Length,
                               Message,
                               SCSI.Status,
                               Execution.Status)
                --}}}
              --}}}
            2
              --{{{  Data Out
              [1]BYTE Null.Buff.1:
              BOOL Success:
              SEQ
                Success := FALSE
                Direction := Host.To.SCSI
                Clear.Screen(fs, ts)
                Display.Line(fs, ts, "Data Transfer Length [0..2^31-1] ?")
                WHILE NOT Success
                  SEQ
                    so.read.echo.any.int(fs, ts, Data.Transfer.Length, Success)
                    IF
                      Data.Transfer.Length < 0
                        Success := FALSE
                      TRUE
                        SKIP
                --{{{  Edit Data
                []BYTE Data RETYPES [Data.Block FROM 0 FOR Data.Transfer.Length]:
                SEQ
                  Hex.Editor(fs, ts, 0,
                             Data,
                             "Edit data to be written to the device")
                  --{{{  Transfer Command to SCSI Device
                  SCSI.HADD.IF ( tscsi,
                                 fscsi,
                                 BYTE Target.ID,
                                 BYTE LUN,
                                 BYTE Command.Length,
                                 SCSI.Command,
                                 Direction,
                                 Zero.32,
                                 Null.Buff.1,
                                 INT32 Data.Transfer.Length,
                                 Data,
                                 Msg.Length,
                                 Message,
                                 SCSI.Status,
                                 Execution.Status)
                  --}}}
                --}}}
              --}}}
            3
              --{{{  Data In
              [1]BYTE Null.Buff.1:
              BOOL Success:
              SEQ
                Success := FALSE
                Direction := SCSI.To.Host
                Clear.Screen(fs, ts)
                Display.Line(fs, ts, "Data Transfer Length [0..2^31-1] ?")
              
                WHILE NOT Success
                  SEQ
                    so.read.echo.any.int(fs, ts, Data.Transfer.Length, Success)
                    IF
                      Data.Transfer.Length < 0
                        Success := FALSE
                      TRUE
                        SKIP
                --{{{  Display Data
                []BYTE Data RETYPES [Data.Block FROM 0 FOR Data.Transfer.Length]:
                SEQ
                  --{{{  Transfer Command to SCSI Device
                  SCSI.HADD.IF ( tscsi,
                                 fscsi,
                                 BYTE Target.ID,
                                 BYTE LUN,
                                 BYTE Command.Length,
                                 SCSI.Command,
                                 Direction,
                                 INT32 Data.Transfer.Length,
                                 Data,
                                 Zero.32,
                                 Null.Buff.1,
                                 Msg.Length,
                                 Message,
                                 SCSI.Status,
                                 Execution.Status)
                  --}}}
                  Hex.Editor(fs, ts, 0,
                             Data,
                             "Data Read From Device")
                --}}}
              --}}}
            ELSE
              SKIP
          --}}}
    --}}}
    --}}}
:
