/*{{{  Header*/
/****************************************************************************\
*                                                                            *
*                         IMS F002 - C libraries                             *
*                                                                            *
*                             Test program                                   *
*                                                                            *
*                    Paul Fellows - Inmos December 1990                      *
*                                                                            *
\****************************************************************************/
/*}}}  */
/*{{{  Include files*/
#include <stdio.h>
#include <process.h>
#include <channel.h>
#include <misc.h>
#include <stdlib.h>
#include <iocntrl.h>

/*{{{  IMS F002C Libraries*/
/*{{{  Those needed for compilation of this program*/
#include <scsi.h>
#include <scsicoms.h>
#include <i_alldev.h>
#include <i_dirtac.h>
/*}}}  */

/*{{{  Other IMS F002C libs NOT USED by this program*/
#include <i_seqacc.h>
#include <tgtmode.h>
/*}}}  */
/*}}}  */

/*}}}  */
/*{{{  Define constants*/
#define buff_size 2048
#define block_size 512
#define number_of_blocks 4

#define TRUE 1    /* Boolean Logical Values */
#define FALSE 0
#define message_array_size 255
#define rx_data_array_size 255
#define tx_data_array_size 255
#define parameter_data_array_size 255
#define parameter_list_array_size 255
#define inquiry_data_array_size 255
#define request_sense_data_array_size 255
#define diagnostic_results_array_size 255
#define device_buffer_array_size 255
#define defect_list_array_size 255
#define read_capacity_data_array_size 255
#define extent_list_array_size 255
#define verification_data_array_size 255
#define scsi_command_array_size 255
#define buffer_array_size 0x100000

/*}}}  */
/*{{{  Scsidemo*/
/*{{{  Local procedures*/
/*{{{  Display execution status*/
void display_execution_status ( short execution_status )

/* Prints current value of execution status variable */

{
  printf ("\n***************************************************");
  printf ("\n Execution Status is %x",execution_status );
  printf ("\n This corresponds to a status of : ");
  switch ( execution_status )
  {
    /*{{{  Display Appropriate Status*/
    case SCSI_E_Good :
      printf( "Good  ");
      break;
    case SCSI_E_Bad_SCSI_Status :
      printf( "Bad_SCSI_Status");
      break;
    case SCSI_E_Command_Not_Complete :
      printf( "Command_Not_Complete ");
      break;
    case SCSI_E_Incorrect_IO_Process_Number :
      printf( "Incorrect_IO_Process_Number");
      break;
    case SCSI_E_SCSI_Bus_Parity_Error :
      printf( "SCSI_Bus_Parity_Error");
      break;
    case SCSI_E_Data_In_Failure :
      printf( "Data_In_Failure");
      break;
    case SCSI_E_Bad_Data_In_Returned_Packet_Size :
      printf( "Bad_Data_In_Returned_Packet_Size ");
      break;
    case SCSI_E_Data_Out_Failure :
      printf( "Data_Out_Failure  ");
      break;
    case SCSI_E_Bad_Data_Out_Returned_Packet_Size :
      printf( "Bad_Data_Out_Returned_Packet_Size");
      break;
    case SCSI_E_Data_In_Time_Out :
      printf( "Data_In_Time_Out  ");
      break;
    case SCSI_E_Data_Out_Time_Out :
      printf( "Data_Out_Time_Out ");
      break;
    case SCSI_E_Command_Time_Out :
      printf( "Command_Time_Out  ");
      break;
    case SCSI_E_Status_Time_Out :
      printf( "Status_Time_Out");
      break;
    case SCSI_E_Controller_Reset_Time_Out :
      printf( "Controller_Reset_Time_Out  ");
      break;
    case SCSI_E_Bus_Reset_Time_Out :
      printf( "Bus_Reset_Time_Out");
      break;
    case SCSI_E_Reselected :
      printf( "Reselected  ");
      break;
    case SCSI_E_Function_Not_Complete :
      printf( "Function_Not_Complete");
      break;
    case SCSI_E_Disconnected :
      printf( "Disconnected");
      break;
    case SCSI_E_Not_Bus_Service :
      printf( "Not_Bus_Service");
      break;
    case SCSI_E_Sequence_Not_Complete :
      printf( "Sequence_Not_Complete");
      break;
    case SCSI_E_Premature_Phase_Change :
      printf( "Premature_Phase_Change  ");
      break;
    case SCSI_E_Target_Did_Not_Assert_ATN :
      printf( "Target_Did_Not_Assert_ATN  ");
      break;
    case SCSI_E_Target_Did_Not_Assert_Message_Out :
      printf( "Target_Did_Not_Assert_Message_Out");
      break;
    case SCSI_E_Selection_Time_Out :
      printf( "Selection_Time_Out");
      break;
    case SCSI_E_Undocumented_Error :
      printf( "Undocumented_Error");
      break;
    case SCSI_E_Data_In_Abort :
      printf( "Data_In_Abort  ");
      break;
    case SCSI_E_Data_Out_Abort :
      printf( "Data_Out_Abort ");
      break;
    case SCSI_E_Unexpected_Phase_Change :
      printf( "Unexpected_Phase_Change ");
      break;
    case SCSI_E_Failed_Self_Test :
      printf( "Failed_Self_Test  ");
      break;
    case SCSI_E_Bad_Interrupt_Time_Out :
      printf( "Interrupt_Time_Out  ");
      break;
    case SCSI_E_Software_Error :
      printf( "Software_Error ");
      break;
    case SCSI_E_Not_Implemented :
      printf( "Not_Implemented");
      break;
    default :
      printf( "Unrecognised Execution Status");
    /*}}}  */
  }
  printf ("\n***************************************************");
  printf ("\n\n");
}

/*}}}  */
/*{{{  Check execution status*/
int check_execution_status ( short execution_status )

/*             Returns TRUE if no error found !                   */

{
  if ( execution_status != SCSI_E_Good )
  {
    display_execution_status ( execution_status );
    return ( FALSE );
  }
  display_execution_status ( execution_status );
  return ( TRUE );
}
/*}}}  */
/*{{{  Display request sense data*/
void display_request_sense_data ( char rs_buffer[] )

/* Displays request sense data passed in as array rs_buffer */

{
  /*{{{  Local variables*/
  int i ;
  /*}}}  */
  /*{{{  Header*/
  printf("\n******************************************************");
  printf("\n                REQUEST SENSE DATA                   \n");
  /*}}}  */
  /*{{{  Fields*/
  printf("    Error code     : %x\n", (rs_buffer[0] & Request_Sense_6_Error_Code));
  printf("    Valid          : %x\n", (((int) rs_buffer[0] >> 7 ) & Request_Sense_6_Valid ));
  printf("    Segment number : %x\n", ( rs_buffer[1] ));
  printf("    Sense key      : %x", ( rs_buffer[2] & Request_Sense_6_Sense_Key ));
  /*}}}  */
  /*{{{  Tail*/
  printf("\n******************************************************");
  printf("\n\n");
  /*}}}  */
  /*{{{  Clear rs_buffer*/
  for ( i=0 ; i < (sizeof rs_buffer) ; i += 1 )
  {
    rs_buffer[i] = 0 ;
  }
  /*}}}  */
}





/*}}}  */
/*{{{  get keypress*/
void get_keypress ()

/* Simply asks for key press */

{
  int key ;
  printf ("  -  Please press any key\n");
  key = getkey ();
}

/*}}}  */
/*{{{  Display inquiry data*/
void display_inquiry_data ( char iq_buffer[] )

/* Displays data received after inquiry command to scsi bus device */

{
  /*{{{  Variables*/
  int peripheral_qualifier ;
  int count ;
  /*}}}  */
  /*{{{  Header*/
  printf ("\n*********************************************************");
  printf ("\n                     INQUIRY DATA\n");
  /*}}}  */
  /*{{{  Fields*/
  /*{{{  Device type*/
  printf ("\n         Peripheral device type        : %x", ((int) iq_buffer[0]
          & Inquiry_6_Peripheral_Device_Type));
  /*}}}  */
  /*{{{  Peripheral qualifier*/
  printf ("\n         Peripheral qualifier          : ");
  
  peripheral_qualifier = (int) iq_buffer[0] ;
  peripheral_qualifier = (( peripheral_qualifier >> 5 ) &
    ( Inquiry_6_Peripheral_Qualifier ));
  printf ("%x", peripheral_qualifier );
  /*}}}  */
  /*{{{  Device type qualifier*/
  printf ("\n         Device type qualifier         : %x", (((int) iq_buffer[1]) &
      Inquiry_6_Device_Type_Qualifier ));
  /*}}}  */
  /*{{{  Removable medium*/
  printf ("\n         Removable medium              : %x", (((int) iq_buffer[1] >> 7) &
            Inquiry_6_RMB ));
  /*}}}  */
  /*{{{  ANSI approved version*/
  printf ("\n         ANSI-approved version         : %x", ((int) iq_buffer[2] &
      Inquiry_6_ANSI_Approved_Version ));
  /*}}}  */
  /*{{{  ECMA version*/
  printf ("\n         ECMA version                  : %x",(((int) iq_buffer[2] >> 3 ) &
      Inquiry_6_ECMA_Version ));
  /*}}}  */
  /*{{{  IOS version*/
  printf ("\n         ISO version                   : %x", (((int) iq_buffer[2] >> 6 ) &
      Inquiry_6_ISO_Version ));
  /*}}}  */
  /*{{{  Response data format*/
  printf ("\n         Response data format          : %x", ((int) iq_buffer[3] &
      Inquiry_6_Response_Data_Format ));
  /*}}}  */
  /*{{{  Terminate I/O process support*/
  printf ("\n         Terminate I/O process support : %x", (((int) iq_buffer[3] >> 6 ) &
      Inquiry_6_TrmIOP ));
  /*}}}  */
  /*{{{  AENC*/
  printf ("\n         AENC                          : %x", (((int) iq_buffer[3] >> 7 ) & Inquiry_6_AENC ));
  /*}}}  */
  /*{{{  Additional length*/
  printf ("\n         Additional length             : %x", (int) iq_buffer[4] );
  /*}}}  */
  /*{{{  Byte 5*/
  printf ("\n         Byte 5                        : %x", (int) iq_buffer[5] );
  /*}}}  */
  /*{{{  Byte 6*/
  printf ("\n         Byte 6                        : %x", (int) iq_buffer[6] );
  /*}}}  */
  /*{{{  Byte 7*/
  printf ("\n         Byte 7                        : %x", (int) iq_buffer[7] );
  /*}}}  */
  /*{{{  Vendor ID*/
  printf ("\n         Vendor ID                     : ");
  
  for ( count = 8 ; count < 16 ; count += 1 )
    putchar ( iq_buffer[count]);
  
  /*}}}  */
  /*{{{  Product ID*/
  printf ("\n         Product Identification        : ");
  for ( count = 16 ; count < 32 ; count += 1 )
    putchar (iq_buffer[count]);
  /*}}}  */
  /*{{{  Product revision level*/
  printf ("\n         Product revision level        : ");
  for ( count = 32 ; count < 35 ; count += 1 )
    putchar (iq_buffer[count]);
  
  /*}}}  */
  /*}}}  */
  /*{{{  Tail*/
  printf ("\n*********************************************************\n");
  printf ("\n\n");
  /*}}}  */
  /*{{{  Clear iq_buffer*/
  for ( count = 0 ; count < (sizeof iq_buffer) ; count += 1 );
    iq_buffer [ count ] = 0 ;
  /*}}}  */
}









/*}}}  */
/*}}}  */
/*{{{  Void scsidemo*/
void scsidemo ( Channel *to_scsi_ha , Channel *from_scsi_ha )

/*         Test program - talks to SCSI tram                  */

{
  /*{{{  Constants*/
  const char   initiator_id             = 7    ;
  const char   target_id                = 0    ;
  const char   inquiry_data_length      = 0x24 ;
  const int    request_sense_allocation = 18   ;
  /*}}}  */
  /*{{{  Variables*/
  char   block[buff_size]                                  ;
  char   message[message_array_size]                       ;
  char   inquiry_data[inquiry_data_array_size]             ;
  char   request_sense_data[request_sense_data_array_size] ;
  
  char   scsi_status , msg_length                          ;
  char   dpo = 0                                           ;
  char   fua = 0                                           ;
  char   lun = 0                                           ;
  char   reladr = 0                                        ;
  short  execution_status                                  ;
  int    logical_block_address                             ;
  
  int    no_error_found = TRUE                             ;
  
  int    i                                                 ;
  int    time_start , time_now                             ;
  int    errors_detected = 0                               ;
  int    old_errors_detected = 0                           ;
  /*}}}  */
  /*{{{  Main program*/
  /*{{{  Title introduction*/
  printf ("\n*******************************************************************");
  printf ("\n*                                                                 *");
  printf ("\n*               TEST PROGRAM FOR F002C 'C' Interfaces             *");
  printf ("\n*                                                                 *");
  printf ("\n*            Target Id (Drive) = 0 : Initiator Id (B422) = 7      *");
  printf ("\n*                                                                 *");
  printf ("\n*                  Inmos iq Systems May 1991                      *");
  printf ("\n*                                                                 *");
  printf ("\n*******************************************************************");
  printf ("\n\n\n\n");
  
  /*}}}  */
  /*{{{  Initialize controller and reset bus*/
  /*{{{  prompt*/
  printf("\nInitialising bus");
  get_keypress ();
  /*}}}  */
  /*{{{  scsi_initialise*/
  SCSI_Initialise ( to_scsi_ha ,
                    from_scsi_ha ,
                    (short int ) Default_Data_Phase_Time_Out ,
                    (short int ) Default_Interrupt_Time_Out ,
                    Enable_Parity_Generation ,
                    Enable_Parity_Checking ,
                    Fast_Cable_Mode ,
                    Reset_SCSI_Controller ,
                    Do_Reset_SCSI_Bus ,
                    Do_Self_Test,
                    &execution_status);
  
  /*}}}  */
  /*{{{  1/2 second delay*/
  time_start = ProcTime ();
  
  while (( time_now = ProcTime ()) < ( time_start + 7813 ))
  {}
  /*}}}  */
  /*}}}  */
  /*{{{  Check execution status*/
  no_error_found = check_execution_status ( execution_status );
  /*}}}  */
  /*{{{  Enable Initiator mode*/
  if ( no_error_found )
  {
    /*{{{  prompt*/
    printf ("\nEnableing initiator mode");
    get_keypress();
    /*}}}  */
    /*{{{  enable initiator mode*/
           Enable_Initiator_Mode (
           to_scsi_ha ,
           from_scsi_ha ,
           initiator_id,
           &execution_status);
    
    
    no_error_found = check_execution_status ( execution_status );
    /*}}}  */
  }
  
  /*}}}  */
  /*{{{  Define data transfer mode*/
  if ( no_error_found )
  {
    /*{{{  prompt*/
    printf ("\nDefining data transfer mode");
    get_keypress ();
    /*}}}  */
    /*{{{  scsi_define_data_transfer_mode*/
    SCSI_Define_Data_Transfer_Mode ( to_scsi_ha ,
                                     from_scsi_ha ,
                                     SCSI_Fast_DMA_Mode,
                                     &execution_status);
    no_error_found = check_execution_status ( execution_status );
    /*}}}  */
  }
  
  
  /*}}}  */
  /*{{{  Scsi request sense 6*/
  if ( no_error_found )
  {
    /*{{{  prompt*/
    printf("\nRequesting sense");
    get_keypress ();
    /*}}}  */
    /*{{{  request_sense_6*/
    msg_length = 0 ;
    /*{{{  call request sense*/
    SCSI_Request_Sense_6 ( to_scsi_ha ,
                           from_scsi_ha ,
                           target_id ,
                           lun ,
                           request_sense_allocation ,
                           Non_Linked_Command ,
                           request_sense_data ,
                           sizeof(request_sense_data),
                           &msg_length ,
                           message ,
                           sizeof(message),
                           &scsi_status,
                           &execution_status);
    
    no_error_found = check_execution_status ( execution_status );
    /*}}}  */
    /*}}}  */
    /*{{{  check_execution_status*/
    no_error_found = check_execution_status ( execution_status );
    /*}}}  */
    /*{{{  display_request_sense_data*/
    display_request_sense_data ( request_sense_data );
    /*}}}  */
  }
  
  /*}}}  */
  /*{{{  Scsi inquiry 6*/
  if ( no_error_found )
  {
    /*{{{  prompt*/
    printf ("\nInquiring after drive");
    get_keypress ();
    /*}}}  */
    /*{{{  scsi_inquiry_6*/
    msg_length = 0 ;
    /*{{{  scsi_inquiry_6*/
    SCSI_Inquiry_6 ( to_scsi_ha ,
            from_scsi_ha ,
            target_id ,
            lun ,
            0 ,
            0 ,
            inquiry_data_length ,
            Non_Linked_Command ,
            inquiry_data ,
            sizeof(inquiry_data),
            &msg_length ,
            message ,
            sizeof(message),
            &scsi_status,
            &execution_status);
    /*}}}  */
    /*}}}  */
    /*{{{  check execution_status*/
    no_error_found = check_execution_status ( execution_status );
    /*}}}  */
    /*{{{  display_inquiry_data*/
    display_inquiry_data ( inquiry_data );
    /*}}}  */
  }
  /*}}}  */
  /*{{{  Define simple test pattern and write to bus*/
  if ( no_error_found )
  {
    /*{{{  set-up write data*/
    for ( i=0 ; i < buff_size ; i += 1 )
    {
      block[i] = ( (char) i/2 );
    }
    /*}}}  */
    /*{{{  prompt*/
    printf("\nWriting to disk");
    get_keypress ();
    /*}}}  */
    /*{{{  scsi_write_10*/
    logical_block_address = 0 ;
    msg_length = 0 ;
    /*{{{  scsi write 10*/
    printf ("\nWriting to drive");
    
    SCSI_Write_10 ( to_scsi_ha ,
           from_scsi_ha ,
           target_id ,
           lun ,
           dpo ,
           fua ,
           reladr ,
           logical_block_address ,
           number_of_blocks ,
           block_size ,
           Non_Linked_Command ,
           block ,
           sizeof(block),
           &msg_length ,
           message ,
           sizeof(message),
           &scsi_status,
           &execution_status);
    
    printf ("\nWritten to drive");
    /*}}}  */
    /*}}}  */
    /*{{{  check_execution_status*/
    no_error_found = check_execution_status ( execution_status );
    /*}}}  */
  }
  /*}}}  */
  /*{{{  Clear buffer , and read back data*/
  if ( no_error_found )
  {
    /*{{{  Clear written data from buffer*/
    printf ("\nClearing buffer");
    for ( i=0 ; i < buff_size ; i += 1 )
    {
      block[i] = 0 ;
    }
    /*}}}  */
    /*{{{  prompt*/
    printf("\nReading from disk");
    get_keypress ();
    /*}}}  */
    /*{{{  scsi read 10*/
    logical_block_address = 0 ;
    msg_length = 0 ;
    SCSI_Read_10 (  to_scsi_ha ,
                    from_scsi_ha ,
                    target_id ,
                    lun ,
                    dpo ,
                    fua ,
                    reladr ,
                    logical_block_address ,
                    number_of_blocks ,
                    block_size ,
                    Non_Linked_Command ,
                    block ,
                    sizeof(block),
                    &msg_length ,
                    message ,
                    sizeof(message),
                    &scsi_status,
                    &execution_status);
    /*}}}  */
    /*{{{  check_execution_status*/
    no_error_found = check_execution_status ( execution_status );
    /*}}}  */
  }
  
  
  
  /*}}}  */
  /*{{{  Check read in data = written data*/
  if ( no_error_found )
  {
    /*{{{  prompt*/
    printf("\nChecking errors");
    get_keypress ();
    /*}}}  */
    /*{{{  display data and check for errors*/
    for ( i=0 ; i < buff_size ; i += 1 )
    {
      /*{{{  if error detected add to count*/
      if ( block[i] != ( (char) i/2 ))
      {
        old_errors_detected = errors_detected;
        errors_detected += 1 ;
      }
      /*}}}  */
      /*{{{  display read data with correct values expected*/
      if (errors_detected > old_errors_detected)
           /*{{{  print the error*/
           {
             printf ("\n            Block[%d] = %d",i,block[i]);
             printf ("       Actual value = %d" , ((char) i/2 ));
           }
           /*}}}  */
      /*}}}  */
    }
    /*}}}  */
    /*{{{  if errors detected , inform user , else congratulate !*/
    if ( errors_detected )
    {
      printf ( "\n\nERROR : Data read does not equal data written !");
      printf ( "\n\nDetected %d errors\n",errors_detected );
    }
    else
    {
      printf ("\n\n No data errors detected !");
    }
    /*}}}  */
  }
  /*}}}  */
  /*{{{  If error found , display scsi status*/
  if ( no_error_found == FALSE )
  {
    /*{{{  error found , request sense*/
    /*{{{  display that error has been found*/
    printf ("\n ERROR DETECTED BY SCSI DEVICE \n");
    printf ("\n Requesting sense information \n");
    /*}}}  */
    /*{{{  request_sense_6*/
    msg_length = 0 ;
    /*{{{  call request sense*/
    SCSI_Request_Sense_6 ( to_scsi_ha ,
           from_scsi_ha ,
           target_id ,
           lun ,
           request_sense_allocation ,
           Non_Linked_Command ,
           request_sense_data ,
           sizeof(request_sense_data),
           &msg_length ,
           message ,
           sizeof(message),
           &scsi_status,
           &execution_status );
    /*}}}  */
    /*}}}  */
    /*{{{  display_request_sense*/
    printf ("\n\n");
    printf ("\nSCSI status = %x \n", scsi_status );
    printf ("\nMessage length = %d \n",msg_length );
    printf ("\nMessage = ");
    for ( i=0 ; i < msg_length ; i += 1 )
    {
      putchar ( message[i] );
    }
    printf ("\n");
    printf ("\nExecution_status = %x \n", execution_status );
    display_request_sense_data ( request_sense_data );
    /*}}}  */
    /*{{{  terminate not ok*/
    printf ("\n\n TERMINATING NOT OK !!\n\n ");
    /*}}}  */
    /*}}}  */
  }
  else
    /*{{{  terminate ok*/
    printf ("\n\n TERMINATING OK !! \n\n");
    /*}}}  */
  
  /*}}}  */
  /*}}}  */
}
/*}}}  */
/*}}}  */
/*{{{  Main entry*/
int main ()
{
  /*{{{  Variables*/
  Channel *to_scsi_ha ;
  Channel *from_scsi_ha ;
  /*}}}  */
  /*{{{  Initialise channels*/
  to_scsi_ha   = ( Channel *) get_param (3) ;   /* Parameters declared at     */
  from_scsi_ha = ( Channel *) get_param (4) ;   /* configuration level.       */
  /*}}}  */
  /*{{{  call scsidemo*/
  scsidemo ( to_scsi_ha ,
      from_scsi_ha ) ;
  /*}}}  */
  /*{{{  terminate*/
  exit_terminate ( EXIT_SUCCESS );
  /*}}}  */
}
/*}}}  */
