/*{{{  Header*/
/*#COMMENT "Copyright INMOS Limited 1990, 1991, 1992" */
/*#COMMENT "IMS F002A Product release SCSI Software libraries" */
/*------------------------------------------------------------------------------ */
/* */
/* Date        : 29-Nov-1990 */
/* */
/* File Name   : SCSICONS.H */
/* */
/* Project     : F002 - SCSI Tram Software */
/* */
/* Version     : 1.2 */
/* */
/* Division    : IQ Systems */
/* */
/* Limitations : This software may be distributed in source form. */
/* */
/* Description : General Constant definitions used throughout IMS F002 */
/*               They must NOT be altered, as these definitions */
/*               are used by the supplied COMPILED Libraries. Failure */
/*               to comply will result in inconsistent definitions between */
/*               the users application program and the supplied library */
/*               modules. */
/* */
/*-------------------------------------------------------------------------- */
/* Schedule of Amendments */
/*-------------------------------------------------------------------------- */
/* 10-May-1990 P.F Removed unused definitions */
/*-------------------------------------------------------------------------- */
/* 17-Aug-1990 P.F Added Interrupt time out default */
/*-------------------------------------------------------------------------- */
/*}}}  */
/*{{{  Constant Definitions*/
/*{{{  #defines*/
/*{{{  Data Transfer Mode */
#define SCSI_Fast_DMA_Mode       1
#define SCSI_Slow_DMA_Mode       2
#define SCSI_Very_Slow_DMA_Mode  3
/*}}}  */
/*{{{  General*/
#define Zero_B 0x00
#define Asserted_B 0xff
/*}}}  */

/*}}}  */
/*{{{  Software ID */
const short int SCSI_Software_ID = 1 ;
/*}}}  */
/*{{{  I.Direction */
const char Non_Data_Transfer                   = 0x00 ;
const char Host_To_SCSI                        = 0x01 ;
const char SCSI_To_Host                        = 0x02 ;
const char Target_To_SCSI                      = 0x03 ;
const char Scsi_To_Target                      = 0x04 ;
/*}}}  */
/*{{{  General */
const int  Low_Byte                               = 0x00ff ;
const int  High_Byte                              = 0xff00 ;
const int  Lp_1_Second                            = 15625 ;
/*}}}  */
/*{{{  Control Byte */
const char Non_Linked_Command          = 0x00;
/*}}}  */
/*{{{  Operating Mode */
const int  Initiator = 0x00 ;
const int  Target    = 0x01 ;
/*}}}  */
/*{{{  Link Management */
const char Single_Link                            = 0x00 ;
const char Double_Link_l_Plus_1                   = 0x01 ;
const char Double_Link_l_Plus_2                   = 0x02 ;
const char Double_Link_l_Plus_3                   = 0x03 ;
const int  Max_Link_Message_Size                  = 256   ;
const int  Link_Count                             = 4     ;
/*}}}  */
/*{{{  Bus Connection State */
const char Disconnected_State = 0x00;
const char Connected_State    = 1 ;
/*}}}  */
/*{{{  Buffer Sizes */
const int  Max_SCSI_Buffer_Size = 16384 ;
const int  Min_SCSI_Buffer_Size = 512 ;
/*}}}  */
/*{{{  Packet Management */
const short int Max_SCSI_Packet_Size               = 8192 ;
const short int Min_SCSI_Packet_Size               = 256  ;
/*}}}  */
/*{{{  SCSI Protocol Sizes */
const int  Max_SCSI_Command_Size                 = 256 ;
const int  Max_SCSI_Message_Size                 = 256 ;
const int  Max_SCSI_IO_Processes                 = 256 ;
const int  Max_SCSI_LUN                          = 7   ;
const int  Max_SCSI_Device_ID                    = 7   ;
/*}}}  */
/*{{{  B422 Watch Dog Timer Defaults */
/* These 2 constants together define the time out period of an information */
/* transfer period (Data In or Data Out SCSI Phase) */
/* */
const int  Default_Ticks_Per_Loop      = 15625;
const int  Default_Data_Phase_Time_Out = 10   ;
/* This constant defines the time to wait for an interrupt from the SCSI */
/* controller chip. */
const int  Default_Interrupt_Time_Out  = 1000 ;
/*}}}  */
/*{{{  Initialisation */
const char Enable_Parity_Checking       = 0xff;
const char Disable_Parity_Checking      = 0x00;
const char Enable_Parity_Generation     = 0xff;
const char Disable_Parity_Generation    = 0x00;
const char Fast_Cable_Mode              = 0xff;
const char Slow_Cable_Mode              = 0x00;
const char Reset_SCSI_Controller        = 0xff;
const char Do_Not_Reset_SCSI_Controller = 0x00;
const char Do_Reset_SCSI_Bus            = 0xff;
const char Do_Not_Reset_SCSI_Bus        = 0x00;
const char Do_Self_Test                 = 0xff;
const char Do_Not_Self_Test             = 0x00;
/*}}}  */
/*{{{  Target Mode*/
const char Do_Not_Ignore_Bus_Reset = 0x00;
const char Ignore_Bus_Reset        = 0xff;
/*}}}  */
/*}}}  */
