--{{{  Header
#COMMENT "Copyright INMOS Limited 1991, 1992"
#COMMENT "IMS F002C Product release SCSI Software libraries"
--------------------------------------------------------------------------------
--
-- Date        : 17-Aug-1990
--
-- File Name   : SCSICONS.INC
--
-- Project     : F002 - SCSI Tram Software
--
-- Version     : 1.2
--
-- Division    : IQ Systems
--
-- Limitations : This software may be distributed in source form.
--
-- Description : General Constant definitions used throughout IMS F002
--               They must NOT be altered, as these definitions
--               are used by the supplied COMPILED Libraries. Failure
--               to comply will result in inconsistent definitions between
--               the users application program and the supplied library
--               modules.
--
----------------------------------------------------------------------------
-- Schedule of Amendments
----------------------------------------------------------------------------
-- 10-May-1990 P.F Removed unused definitions
----------------------------------------------------------------------------
-- 17-Aug-1990 P.F Added Interrupt time out default
----------------------------------------------------------------------------
--}}}  
--{{{  Constant Definitions
--{{{  Software ID
VAL INT16 SCSI.Software.ID IS 1 (INT16):
--}}}  
--{{{  I.Direction
VAL BYTE Non.Data.Transfer                   IS #00(BYTE):
VAL BYTE Host.To.SCSI                        IS #01(BYTE):
VAL BYTE SCSI.To.Host                        IS #02(BYTE):
VAL BYTE Target.To.SCSI                      IS #03(BYTE):
VAL BYTE SCSI.To.Target                      IS #04(BYTE):

VAL BYTE Max.SCSI.Valid.Direction            IS #04(BYTE):
--}}}  
--{{{  General
VAL BYTE  Zero.B                                 IS #00 (BYTE):
VAL INT16 Zero.16                                IS #00 (INT16):
VAL INT32 Zero.32                                IS #00 (INT32):
VAL BYTE  Negated.B                              IS #00 (BYTE):
VAL BYTE  Asserted.B                             IS #FF (BYTE):
VAL INT   Low.Byte                               IS #00FF :
VAL INT   High.Byte                              IS #FF00 :
VAL INT   LP.1.Second                            IS 15625 :
--}}}  
--{{{  Control Byte
VAL BYTE Non.Linked.Command          IS Zero.B:
--}}}  
--{{{  Operating Mode
VAL INT Initiator IS #00 :
VAL INT Target    IS #01 :
--}}}  
--{{{  Data Transfer Mode
VAL INT16 SCSI.Fast.DMA.Mode      IS  1 (INT16):
VAL INT16 SCSI.Slow.DMA.Mode      IS  2 (INT16):
VAL INT16 SCSI.Very.Slow.DMA.Mode IS  3 (INT16):
--}}}  
--{{{  Link Management
VAL BYTE Single.Link                            IS #00 (BYTE):
VAL BYTE Double.Link.L.Plus.1                   IS #01 (BYTE):
VAL BYTE Double.Link.L.Plus.2                   IS #02 (BYTE):
VAL BYTE Double.Link.L.Plus.3                   IS #03 (BYTE):
VAL INT Max.Link.Message.Size                   IS 256   :
VAL INT Link.Count                              IS 4     :
--}}}  
--{{{  Bus Connection State
VAL BYTE Disconnected.State IS Zero.B:
VAL BYTE Connected.State    IS 1 (BYTE):
--}}}  
--{{{  Buffer Sizes
VAL INT Max.SCSI.Buffer.Size IS 16384 : -- Max Size of Buffer on B422 TRAM
VAL INT Min.SCSI.Buffer.Size IS 512 :   -- Min Size of Buffer on B422 TRAM
--}}}  
--{{{  Packet Management
VAL INT16 Max.SCSI.Packet.Size               IS 8192 (INT16) :
VAL INT16 Min.SCSI.Packet.Size               IS 256  (INT16) :
--}}}  
--{{{  SCSI Protocol Sizes
VAL INT  Max.SCSI.Command.Size                 IS 256 :
VAL INT  Max.SCSI.Message.Size                 IS 256 :
VAL INT  Max.SCSI.IO.Processes                 IS 256 :
VAL INT  Max.SCSI.LUN                          IS 7   :
VAL INT  Max.SCSI.Device.ID                    IS 7   :
--}}}  
--{{{  B422 Watch Dog Timer Defaults
-- These 2 constants together define the time out period of an information
-- transfer period (Data In or Data Out SCSI Phase)
--
VAL INT Default.Ticks.Per.Loop      IS 15625: --T2 => 1  Second
VAL INT Default.Data.Phase.Time.Out IS 10   : --T2 => 10 Seconds
-- This constant defines the time to wait for an interrupt from the SCSI
-- controller chip.
VAL INT Default.Interrupt.Time.Out  IS 1000 : --T2 => 1  Second
--}}}  
--{{{  Initialisation
VAL BYTE Enable.Parity.Checking       IS Asserted.B:
VAL BYTE Disable.Parity.Checking      IS Zero.B:
VAL BYTE Enable.Parity.Generation     IS Asserted.B:
VAL BYTE Disable.Parity.Generation    IS Zero.B:
VAL BYTE Fast.Cable.Mode              IS Asserted.B:
VAL BYTE Slow.Cable.Mode              IS Zero.B:
VAL BYTE Reset.SCSI.Controller        IS Asserted.B:
VAL BYTE Do.Not.Reset.SCSI.Controller IS Zero.B:
VAL BYTE Do.Reset.SCSI.Bus            IS Asserted.B:
VAL BYTE Do.Not.Reset.SCSI.Bus        IS Zero.B:
VAL BYTE Do.Self.Test                 IS Asserted.B:
VAL BYTE Do.Not.Self.Test             IS Zero.B:
--}}}  
--{{{  Target Mode
VAL BYTE Ignore.Bus.Reset        IS Asserted.B:
VAL BYTE Do.Not.Ignore.Bus.Reset IS Zero.B:
--}}}  
--}}}  
