--{{{  Header
#COMMENT "Copyright INMOS Limited 1990, 1991, 1992"
#COMMENT "IMS F002C Product release SCSI Software libraries"
------------------------------------------------------------------------------
--
-- Date        : 22-Aug-1990
--
-- File Name   : SCSIPCOL.INC
--
-- Project     : IMS F002 - SCSI Tram Software
--
-- Division    : IQ Systems
--
-- Version No. : 1.2
--
-- Limitations : This software may be distributed in source form.
--
-- Description : Definitions of link protocols TO.SCSI and FROM.SCSI
--
----------------------------------------------------------------------------
-- Schedule of Amendments
----------------------------------------------------------------------------
-- 25-July-1990 P.F Removed unused definitions
----------------------------------------------------------------------------
-- 22-Aug-1990  P.F Added future expansion hooks
----------------------------------------------------------------------------

--}}}  
--{{{  Protocols
--{{{  TO.SCSI
PROTOCOL TO.SCSI
  CASE
    --{{{  Operating mode
    SCSI.Become.An.Initiatior -- defaults to initiator
    SCSI.Become.A.Target
    --}}}  
    --{{{  Initiator mode operation
    --{{{  Comments
    --------------------------------------------------------------------------
    -- initiator mode operation
    --------------------------------------------------------------------------
    --
    -- In order to facilitate for mulithreaded I/O on each link in addition
    -- to mulitithreading by virtue of using more than one link, the concept
    -- of providing an I/O process number is introduced.
    --
    -- To provide for mutually exclusive job numbers, the I/O process number
    -- for an individual I/O process on a single link is a defined as a
    -- 16 bit Integer (INT16) whose bits have the following significance :-
    --
    --       ----------------------------------------
    --       |15 14 13 12 11 10 9 8 7 6 5 4 3 2 1 0 |
    --       ----------------------------------------
    --       |Queue Tag            |Target ID| LUN  |
    --       ----------------------------------------
    --
    --       Where :
    --
    --        Queue Tag = SCSI-2  5.6.17 Queue Tag Messages
    --        Target ID = SCSI Target ID
    --        LUN       = SCSI Logical Unit
    --
    --       If the target does not implement tagged queuing, then the Queue
    --       Tag field will be invalid and be set to zero. Packets returned
    --       on FROM.SCSI will have the Queue Tag field set to zero in such
    --       circumstances.
    --
    -- The format for each initiator sequence will therefore be :-
    --
    -- SCSI.I.Xx..xx    I/O Process Number; Parameters
    --
    --
    --}}}  
    
    SCSI.I.Target.Id;        INT16; BYTE         -- 0..7 to which target.
    SCSI.I.LUN;              INT16; BYTE         -- 0..7 to which LUN
    SCSI.I.Message.Buffer;   INT16; BYTE::[]BYTE -- messages to target.
    SCSI.I.Command.Buffer;   INT16; BYTE::[]BYTE -- commands to target.
    
    SCSI.I.Direction;        INT16; BYTE         -- direction of data transfer.
    
    --{{{  Comments
    --
    -- SCSI.I.Direction = [ Non.Data.Transfer |
    --                      Host.To.SCSI      |
    --                      SCSI.To.Host      |
    --                      Target.To.SCSI    |
    --                      SCSI.To.Target    ]
    --
    --}}}  
    
    SCSI.I.Start.IO.Process; INT16
    
    SCSI.I.Tx.Length;        INT16; INT32 -- tx data quantity 0..2^63-1
    SCSI.I.Rx.Length;        INT16; INT32 -- rx data quantity 0..2^63-1
    SCSI.I.Tx.Data;          INT16; INT16::[]BYTE -- 1..Tx.Buff.Size of BYTE
    --}}}  
    --{{{  Target mode operation
    SCSI.T.Await.Selection.By.Initiator
    SCSI.T.Disconnect                           -- Disconnect from SCSI bus
    SCSI.T.Reconnect;             BYTE          -- 0..7 Initiator on SCSI bus
    SCSI.T.Data.In;               INT32         -- tx data quantity 0..2^31-1
    SCSI.T.Data.Out;              INT32         -- rx data quantity 0..2^31-1
    SCSI.T.Tx.Data;               INT16::[]BYTE -- 1..Tx.Buff.Size of BYTE
    SCSI.T.Status;                BYTE          -- 0..255
    SCSI.T.Message.In;            BYTE::[]BYTE  -- message to initiator
    SCSI.T.Message.Out                          -- request message from initiator
    SCSI.T.Status.And.Message.In; BYTE;         -- Status
                                  BYTE;         -- Control
                                  BYTE::[]BYTE  -- Message
    --}}}  
    --{{{  General house keeping
    SCSI.S.Define.Id; BYTE -- ID of this SCSI device.
    SCSI.S.Reset.SCSI.Bus
    SCSI.S.Reset.SCSI.Adaptor; BYTE; -- Parity Generation
                               BYTE; -- Parity Checking
                               BYTE  -- Slow.Cable.Mode
    --}}}  
    --{{{  Data Transfer Mode
    SCSI.Data.Transfer.Mode; INT16
    --}}}  
    --{{{  Link Management
    
    --------------------------------------------------------------------------
    -- Link Management
    --------------------------------------------------------------------------
    --
    -- The first link that communicates to the SCSI I/O subsystem after
    -- program start, defines the mode in which the links will continue to
    -- operate for eternity. Subsequent link management requests down any
    -- link will result in a rejection message being sent down the pertanant
    -- link on FROM.SCSI
    --
    -- See SCSICONS.INC for definitions
    
    SCSI.Link.Management; BYTE
    --}}}  
    --{{{  Packet Management
    SCSI.PM.Request.Packet.Size; INT16  -- 0..SCSI.Max.Packet.Size
    --}}}  
    --{{{  Messages to other links
    
    --------------------------------------------------------------------------
    -- Messages to other links
    --------------------------------------------------------------------------
    --
    -- To permit dialogue between transputers connected to the SCSI TRAM, a
    -- facility is provided that enables user defined messages to be
    -- transmitted to other links of the SCSI TRAM.
    --
    -- A Byte following the tag SCSI.Message.To.Other.Link signifies which
    -- link the supplied message is to be routed to. Messages will therefore
    -- appear on the protocol FROM.SCSI on the pertanent link.
    --
    
    SCSI.Message.To.Other.Link; BYTE; INT16::[]BYTE  -- 0..3; Message
    
    --}}}  
    --{{{  Transputer Subsystem control
    SCSI.SS.Reset.Subsystem
    SCSI.SS.Analyse.Subsystem
    SCSI.SS.Read.Subsystem.not.Error
    --}}}  
    --{{{  SCSI Subsystem Control
    SCSI.Perform.Self.Test
    --}}}  
    --{{{  Software ID
    SCSI.SID.Report.Version.Number
    SCSI.SID.Report.Vendor
    --}}}  
    --{{{  Execution Status
    SCSI.E.Request.Execution.Status; INT16 --IO.Pno
    --}}}  
    --{{{  Watch Dog Time Outs
    SCSI.WD.Data.Phase.Time.Out;   INT16 -- Number of 1 second periods.
                                         -- Max time that a single SCSI command
                                         -- is permitted to spend in either a
                                         -- Data Out or Data In phase. Ensure
                                         -- that this period is sufficient to
                                         -- allow to quantity of data specified
                                         -- to be transferred to/from the device
                                         -- respectively. Take into account the
                                         -- transfer rate of the device/b422
                                         -- which ever is the slower.
    
    SCSI.WD.Interrupt.Time.Out; INT16    -- Number of 1 millisecond periods.
                                         -- This time out specifies the period
                                         -- between switching from one phase to
                                         -- another in initiator mode. In Target
                                         -- mode it is the period between the
                                         -- target finishing a sequence and the
                                         -- initiator acknowledging that the
                                         -- sequence has completed.
    --}}}  
    --{{{  Future Expansion
    Generic.Input; INT16::[]BYTE
    --}}}  
:
--}}}  
--{{{  FROM.SCSI
PROTOCOL FROM.SCSI
  CASE
    --{{{  Initiator mode operation
    SCSI.I.Tx.Send.Packet; INT16;        -- IO.Pno
                           INT32;        -- Transfer Count
                           INT16         -- Size of requested packet. Respond
                                         -- with SCSI.I.Tx.Data on TO.SCSI
    
    SCSI.I.Rx.Data;        INT16;        -- IO.Pno
                           INT32;        -- Transfer Count
                           INT16::[]BYTE -- 1..Size.4 of BYTE
    
    SCSI.I.Msgbuff;        INT16;        -- IO.Pno
                           BYTE::[]BYTE  -- messages from target
    
    SCSI.I.Status;         INT16;        -- IO.Pno
                           BYTE          -- 0..255 from target
    --}}}  
    --{{{  Target mode operation
    --                       Host.ID, Bus State, Received Command.
    SCSI.T.Command.Buffer;   BYTE;    BYTE;      BYTE::[]BYTE --
    SCSI.T.Msg.Out.Data;     BYTE::[]BYTE  -- messages from initiator
    SCSI.T.Rx.Data;          INT16::[]BYTE -- 1..Size.4 of BYTE
    --}}}  
    --{{{  Packet Management
    SCSI.PM.Allocated.Packet.Size; INT16
    --}}}  
    --{{{  Message from other links
    --------------------------------------------------------------------------
    -- Messages from other links
    --------------------------------------------------------------------------
    --
    -- To permit dialogue between transputers connected to the SCSI TRAM, a
    -- facility is provided that enables a user defined message to be
    -- transmitted on a TO.SCSI of one link to a FROM.SCSI of another link.
    --
    -- A Byte following the tag SCSI.Message.From.Other.Link signifies which
    -- link the supplied message emanated from.
    --
    
    SCSI.Message.From.Other.Link; BYTE; INT16::[]BYTE -- 0..3 ; Message
    
    --}}}  
    --{{{  Execution Status
    SCSI.E.Execution.Status; INT16; INT16          -- Success of SCSI command
                                                   -- at end of execution.
    -- See "scsiesta.inc" for definitions
    SCSI.DL.End.Double.Link;  INT16
    --}}}  
    --{{{  Transputer Sub System Control
    SCSI.SS.Subsystem.not.Error; BYTE
    --}}}  
    --{{{  Software ID
    SCSI.SID.Version.Number; INT16
    SCSI.SID.Vendor;         BYTE::[]BYTE
    --}}}  
    --{{{  Future Expansion
    Genric.Output; INT16::[]BYTE
    --}}}  
:
--}}}  
--}}}  
