--{{{  Header
--#COMMENT "Copyright INMOS Limited 1990, 1991, 1992"
--#COMMENT "IMS F002C Product release SCSI software libraries"
--------------------------------------------------------------------------------
--
-- Date        : 19-Jan-1990 09:46:20
--
-- File Name   : SCSICP7.INC
--
-- Project     : F002 - SCSI Tram Software
--
-- Version     : 1.0
--
-- Division    : IQ Systems
--
-- Limitations : This software may be distributed in source form.
--
-- Description : Slow tx dma loop ( non GUY coded )
--
--------------------------------------------------------------------------------
-- Schedule of Amendments
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--}}}  
--{{{  implementation
PROC Slow.Tx.DMA.Loop( VAL INT   Packet.Size,
                       VAL INT   Data.Transfer.Mode,
                       VAL INT   Tx.DMA.Buffer.Pointer,
                       INT16     Execution.Status)

  --{{{  Comments
  -- This procedure performs a pseudo DMA operation upon the NCR53c94
  --}}}  
  --{{{  Declarations
  VAL INT Bytes.Per.Word IS 2:
  INT Words, Bytes :
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Determine profile of loop
    Words := Packet.Size  / Bytes.Per.Word
    Bytes := (Packet.Size - (Words * Bytes.Per.Word))
    --}}}  
    --{{{  Words
    IF
      Words > 0
        --{{{  Perform DMA
        --{{{  Declarations
        []INT Tx.Words RETYPES [Tx.Buffer FROM Tx.DMA.Buffer.Pointer FOR (Words << 1)]:
        INT Word.Count:
        --}}}  
        SEQ
          --{{{  Initialisation
          Word.Count := 0
          --}}}  
          --{{{  Perform DMA
          IF
            Data.Transfer.Mode = 0
              --{{{  DMA 2 bytes at a time with no delay
              INT DRQ.Test, One:
              SEQ
                DRQ.Test := 0
                One := 1
                WHILE Word.Count < Words
                  SEQ
                    --{{{  Wait for permission to access DMA port
                    WHILE ((DRQ.Flag /\ One) = DRQ.Test)
                      SEQ
                        CASE B422.Status -- see addrplac.inc
                          B422.Ok
                            SKIP
                          Timed.Out
                            --{{{  Timed Out Exeception Handler
                            SEQ
                              --{{{  Record Error
                              CASE Operating.Mode
                                Initiator
                                  Execution.Status := SCSI.E.Data.Out.Time.Out
                                Target
                                  Execution.Status := SCSI.E.Data.In.Time.Out
                                ELSE
                                  Execution.Status := SCSI.E.Software.Error
                              --}}}  
                              --{{{  force drop out of loops
                              DRQ.Test := 1                     -- force drop out of inner loop
                              One := 0
                              Word.Count := Words - 1 -- force drop out of outer loop
                              --}}}  
                            --}}}  
                          Interrupt.Occurred
                            --{{{  Interrupt Exeception Handler
                            SEQ
                              --{{{  Record Error
                              CASE Operating.Mode
                                Initiator
                                  Execution.Status := SCSI.E.Data.Out.Failure
                                Target
                                  Execution.Status := SCSI.E.Data.In.Failure
                                ELSE
                                  Execution.Status := SCSI.E.Software.Error
                              --}}}  
                              --{{{  force drop out of loops
                              DRQ.Test := 1                     -- force drop out of inner loop
                              One := 0
                              Word.Count := Words - 1 -- force drop out of outer loop
                              --}}}  
                            --}}}  
                          ELSE
                            --{{{  Software Error Exeception Handler
                            SEQ
                              --{{{  Record Error
                              Execution.Status := SCSI.E.Software.Error
                              --}}}  
                              --{{{  force drop out of loops
                              DRQ.Test := 1                     -- force drop out of inner loop
                              One := 0
                              Word.Count := Words - 1 -- force drop out of outer loop
                              --}}}  
                            --}}}  
                    --}}}  
                    --{{{  Access DMA port
                    FIFO.DMA := Tx.Words[Word.Count]
                    Word.Count := Word.Count + 1
                    --}}}  
              --}}}  
            Data.Transfer.Mode > 0
              --{{{  DMA 2 bytes at a time with delay of between 1 and 32767 uSeconds
              INT DRQ.Test, One:
              SEQ
                DRQ.Test := 0
                One := 1
                --{{{  DEBUG
                --}}}  
                WHILE Word.Count < Words
                  SEQ
                    --{{{  Wait for permission to access DMA port
                    WHILE ((DRQ.Flag /\ One) = DRQ.Test)
                      SEQ
                        CASE B422.Status -- see addrplac.inc
                          B422.Ok
                            SKIP
                          Timed.Out
                            --{{{  Timed Out Exeception Handler
                            SEQ
                              --{{{  Record Error
                              CASE Operating.Mode
                                Initiator
                                  Execution.Status := SCSI.E.Data.In.Time.Out
                                Target
                                  Execution.Status := SCSI.E.Data.Out.Time.Out
                                ELSE
                                  Execution.Status := SCSI.E.Software.Error
                              --}}}  
                              --{{{  force drop out of loops
                              DRQ.Test := 1                     -- force drop out of inner loop
                              One := 0
                              Word.Count := Words - 1 -- force drop out of outer loop
                              --}}}  
                            --}}}  
                          Interrupt.Occurred
                            --{{{  Interrupt Exeception Handler
                            SEQ
                              --{{{  Record Error
                              CASE Operating.Mode
                                Initiator
                                  Execution.Status := SCSI.E.Data.In.Time.Out
                                Target
                                  Execution.Status := SCSI.E.Data.Out.Time.Out
                                ELSE
                                  Execution.Status := SCSI.E.Software.Error
                              --}}}  
                              --{{{  force drop out of loops
                              DRQ.Test := 1                     -- force drop out of inner loop
                              One := 0
                              Word.Count := Words - 1 -- force drop out of outer loop
                              --}}}  
                            --}}}  
                          ELSE
                            --{{{  Software Error Exeception Handler
                            SEQ
                              --{{{  Record Error
                              Execution.Status := SCSI.E.Software.Error
                              --}}}  
                              --{{{  force drop out of loops
                              DRQ.Test := 1                     -- force drop out of inner loop
                              One := 0
                              Word.Count := Words - 1 -- force drop out of outer loop
                              --}}}  
                            --}}}  
                    --}}}  
                    --{{{  Access DMA port
                    FIFO.DMA := Tx.Words[Word.Count]
                    Word.Count := Word.Count + 1
                    --{{{  Delay for n micro seconds
                    PRI PAR
                      --{{{  Delay
                      INT Now:
                      TIMER Clock:
                      SEQ
                        Clock ? Now
                        Clock ? AFTER (Now PLUS Data.Transfer.Mode)
                      --}}}  
                      SKIP
                    --}}}  
                    --}}}  
              --}}}  
            TRUE
              Execution.Status := SCSI.E.Software.Error
          --}}}  
        --}}}  
      TRUE
        SKIP
    --}}}  
    --{{{  Last Byte if Packet.Size is ODD
    IF
      (Bytes > 0)
        --{{{  If OK then perform DMA
        IF
          (Execution.Status = SCSI.E.Good)
            --{{{  Perform DMA
            INT DRQ.Test:
            SEQ
              --{{{  DEBUG
              --}}}  
              --{{{  Wait for permission to access DMA port
              DRQ.Test := 0
              WHILE ((DRQ.Flag /\ 1) = DRQ.Test)
                SEQ
                  CASE B422.Status -- see addrplac.inc
                    B422.Ok
                      SKIP
                    Timed.Out
                      --{{{  Timed Out Exeception Handler
                      SEQ
                        --{{{  Record Error
                        CASE Operating.Mode
                          Initiator
                            Execution.Status := SCSI.E.Data.Out.Time.Out
                          Target
                            Execution.Status := SCSI.E.Data.In.Time.Out
                          ELSE
                            Execution.Status := SCSI.E.Software.Error
                        --}}}  
                        --{{{  Force drop out of DRQ loop
                        DRQ.Test := 1
                        --}}}  
                      --}}}  
                    Interrupt.Occurred
                      --{{{  Interrupt Exeception Handler
                      SEQ
                        --{{{  Record Error
                        CASE Operating.Mode
                          Initiator
                            Execution.Status := SCSI.E.Data.Out.Failure
                          Target
                            Execution.Status := SCSI.E.Data.In.Failure
                          ELSE
                            Execution.Status := SCSI.E.Software.Error
                        --}}}  
                        --{{{  force drop out of DRQ loop
                        DRQ.Test := 1
                        --}}}  
                      --}}}  
                    ELSE
                      --{{{  Software Error Exeception Handler
                      SEQ
                        --{{{  Record Error
                        Execution.Status := SCSI.E.Software.Error
                        --}}}  
                        --{{{  force drop out of DRQ loop
                        DRQ.Test := 1
                        --}}}  
                      --}}}  
              --}}}  
              --{{{  Access DMA port
              IF
                Execution.Status = SCSI.E.Good
                  FIFO.DMA := INT Tx.Buffer[Packet.Size]
                TRUE
                  SKIP
              --}}}  
            --}}}  
          TRUE
            SKIP
        --}}}  
      TRUE
        SKIP
    --}}}  
  --}}}  
:
--}}}  
